@@ -153,7 +153,7 @@ void *
 grad_list_item(struct list *list, size_t n)
 {
 	struct list_entry *p;
-	if (n >= list->count)
+	if (!list || n >= list->count)
 		return NULL;
 	for (p = list->head; n > 0 && p; p = p->next, n--)
 		;
