@@ -52,16 +52,10 @@ grad_request_alloc()
 void 
 grad_request_free(grad_request_t *radreq)
 {
-	grad_list_destroy(&radreq->locus_list, NULL, NULL);
-	grad_free(radreq->remote_user);
-        grad_avl_free(radreq->reply_pairs);
-        grad_free(radreq->reply_msg);
-        grad_avl_free(radreq->server_reply);
-        grad_avl_free(radreq->request);
+        grad_avl_free(radreq->avlist);
         grad_free(radreq);
 }
 
-
 /* Turn printable string (dictionary type DATE) into correct tm struct entries
  */
 static char *months[] = {
@@ -135,11 +129,11 @@ grad_unlock_file(int fd, size_t size, off_t offset, int whence)
         fcntl(fd, F_SETLKW, &fl);
 }
 
-/* Find a struct keyword matching the given string. Return keyword token
+/* Find a grad_keyword_t matching the given string. Return keyword token
  * number if found. Otherwise return default value `def'.
  */
 int
-grad_xlat_keyword(struct keyword *kw, char *str, int def)
+grad_xlat_keyword(grad_keyword_t *kw, const char *str, int def)
 {
         for ( ; kw->name; kw++) 
                 if (strcmp(str, kw->name) == 0)
@@ -277,7 +271,7 @@ grad_op_to_str(enum grad_operator op)
 enum grad_operator
 grad_str_to_op(char *str)
 {
-        int op = NUM_OPERATORS;
+        int op = grad_operator_invalid;
         switch (*str++) {
         case '=':
                 op = grad_operator_equal;
@@ -300,7 +294,7 @@ grad_str_to_op(char *str)
                 break;
         }
         if (*str)
-                op = NUM_OPERATORS;
+                op = grad_operator_invalid;
         return op;
 }
 
@@ -403,7 +397,7 @@ grad_format_pair(grad_avp_t *pair, int typeflag, char **savep)
 {
         char *buf1 = NULL;
         char *buf2ptr = NULL;
-        char buf2[4*AUTH_STRING_LEN+1]; /* Enough to hold longest possible
+        char buf2[4*GRAD_STRING_LENGTH+1]; /* Enough to hold longest possible
                                            string value all converted to
                                            octal */
         grad_dict_value_t *dval;
@@ -412,8 +406,8 @@ grad_format_pair(grad_avp_t *pair, int typeflag, char **savep)
 	
         *savep = NULL;
 
-        switch (pair->eval_type == eval_const ? pair->type : TYPE_STRING) {
-        case TYPE_STRING:
+        switch (pair->eval_type == grad_eval_const ? pair->type : GRAD_TYPE_STRING) {
+        case GRAD_TYPE_STRING:
                 if (pair->attribute != DA_VENDOR_SPECIFIC) {
                         int len = strlen (pair->avp_strvalue);
                         if (len != pair->avp_strlength-1)
@@ -436,8 +430,8 @@ grad_format_pair(grad_avp_t *pair, int typeflag, char **savep)
                 }
                 break;
                                         
-        case TYPE_INTEGER:
-                if (pair->name && (pair->prop & AP_TRANSLATE))
+        case GRAD_TYPE_INTEGER:
+                if (pair->name && (pair->prop & GRAD_AP_TRANSLATE))
                         dval = grad_value_lookup(pair->avp_lvalue, pair->name);
                 else
                         dval = NULL;
@@ -448,11 +442,11 @@ grad_format_pair(grad_avp_t *pair, int typeflag, char **savep)
                         snprintf(buf2, sizeof(buf2), "%s", dval->name);
                 break;
                 
-        case TYPE_IPADDR:
+        case GRAD_TYPE_IPADDR:
                 grad_ip_iptostr(pair->avp_lvalue, buf2);
                 break;
                 
-        case TYPE_DATE:
+        case GRAD_TYPE_DATE:
                 strftime(buf2, sizeof(buf2), "\"%b %e %Y\"",
                          localtime_r((time_t *)&pair->avp_lvalue, &tm));
                 break;
@@ -462,19 +456,19 @@ grad_format_pair(grad_avp_t *pair, int typeflag, char **savep)
 
 	if (typeflag) {
 		switch (pair->type) {
-		case TYPE_STRING:
+		case GRAD_TYPE_STRING:
 			type = "(STRING) ";
 			break;
 			
-		case TYPE_INTEGER:
+		case GRAD_TYPE_INTEGER:
 			type = "(INTEGER) ";
 			break;
 			
-		case TYPE_IPADDR:
+		case GRAD_TYPE_IPADDR:
 			type = "(IPADDR) ";
 			break;
 			
-		case TYPE_DATE:
+		case GRAD_TYPE_DATE:
 			type = "(DATE) ";
 			break;
 		}
