@@ -1,3 +1,4 @@
+#define EMBEDDED_EXPIRATION_INFO
 /* This file is part of GNU Radius
    Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
@@ -82,6 +83,43 @@ check_user_name(char *p)
 
 LOCK_DECLARE(lock)
 
+#ifdef EMBEDDED_EXPIRATION_INFO	
+static char base64[] =         /* 0 ... 63 => ascii - 64 */
+        "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
+
+static int
+from_base64(u_char *data, int len, unsigned int *value)
+{
+	int i;
+
+	*value = 0;
+	for (i = len-1; i >= 0; i--) {
+		char *p = strchr(base64, data[i]);
+		if (!p)
+			return 1;
+		*value <<= 6;
+		*value += p - base64;
+	}
+	return 0;
+}
+	
+static int
+decode_aging_info(char *p, u_int *maxweeks, u_int *minweeks, u_int *lastchange)
+{
+	if (from_base64(p, 1, maxweeks))
+		return 1;
+	p++;
+	if (from_base64(p, 1, minweeks))
+		return 1;
+	p++;
+	if (from_base64(p, 2, lastchange))
+		return 1;
+	if (p[2])
+		return 1;
+	return 0;
+}
+#endif	
+	
 static enum auth_status
 unix_expiration(char *name, time_t *exp)
 {
@@ -126,6 +164,45 @@ unix_expiration(char *name, time_t *exp)
 		*exp = t;
 # endif
 	}
+#endif
+#ifdef EMBEDDED_EXPIRATION_INFO
+	if (status == auth_ok) {
+		struct passwd *pwd;
+		char *p;
+#define SECS_IN_WEEK 604800
+	
+		if (pwd = getpwnam(name)) {
+			p = strchr(pwd->pw_passwd, ',');
+			if (p) {
+				u_int maxweeks, minweeks, lastchange;
+			
+				if (decode_aging_info(p+1,
+						      &maxweeks,
+						      &minweeks,
+						      &lastchange) == 0) {
+					time_t now = time(NULL);
+					time_t nweeks = now / SECS_IN_WEEK
+						        - lastchange;
+					
+					if (maxweeks == minweeks)
+						return auth_password_expired;
+				
+					if (nweeks >= minweeks
+					    && nweeks >= maxweeks)
+						return auth_password_expired;
+
+					*exp = (maxweeks - nweeks)
+						* SECS_IN_WEEK
+						+ now % SECS_IN_WEEK;
+					status = auth_valid;
+				} else { /* Invalid password data? */
+					grad_log(L_NOTICE,
+						 _("Invalid password aging information for user '%s'"), name);
+					status = auth_fail;
+				}
+			}
+		}
+	}
 #endif
 	return status;
 }
@@ -176,10 +253,18 @@ unix_pass(char *name, char *passwd)
         /*
          * Check encrypted password.
          */
-        pwlen = strlen(encrypted_pass)+1;
-        encpw = grad_emalloc(pwlen);
-        rc = grad_md5crypt(passwd, encrypted_pass, encpw, pwlen) == NULL
-                || strcmp(encpw, encrypted_pass);
+        pwlen = strlen(encrypted_pass);
+        encpw = grad_emalloc(pwlen+1);
+#ifdef EMBEDDED_EXPIRATION_INFO
+ {
+	/* Some systems append expiration data to the encrypted password */
+	char *p = strchr(encrypted_pass, ',');
+	if (p) 
+		pwlen = p - encrypted_pass;
+ }
+#endif
+        rc = grad_md5crypt(passwd, encrypted_pass, encpw, pwlen+1) == NULL
+                || memcmp(encpw, encrypted_pass, pwlen);
         grad_free(encpw);
 	grad_free(encrypted_pass);
         if (rc)
@@ -189,16 +274,17 @@ unix_pass(char *name, char *passwd)
 }
 
 int
-rad_auth_check_username(grad_request_t *radreq, int activefd)
+rad_auth_check_username(radiusd_request_t *radreq, int activefd)
 {
-        grad_avp_t *namepair = grad_avl_find(radreq->request, DA_USER_NAME);
+        grad_avp_t *namepair = grad_avl_find(radreq->request->avlist,
+					     DA_USER_NAME);
 
 	log_open(L_AUTH);
 
         if (grad_avp_null_string_p(namepair)) 
-                grad_log_req(L_ERR, radreq, _("No username"));
+                grad_log_req(L_ERR, radreq->request, _("No username"));
 	else if (check_user_name(namepair->avp_strvalue)) 
-                grad_log_req(L_ERR, radreq, _("Malformed username"));
+                grad_log_req(L_ERR, radreq->request, _("Malformed username"));
 	else
 		return 0;
 
@@ -209,13 +295,13 @@ rad_auth_check_username(grad_request_t *radreq, int activefd)
 				  message_text[MSG_ACCESS_DENIED],
 				  activefd);
 	else
-		stat_inc(auth, radreq->ipaddr, num_bad_req);
+		stat_inc(auth, radreq->request->ipaddr, num_bad_req);
 	return -1;
 }
 
 /* Initial step of authentication. */
 int
-rad_auth_init(grad_request_t *radreq, int activefd)
+rad_auth_init(radiusd_request_t *radreq, int activefd)
 {
 	grad_locus_t loc;
 	
@@ -228,11 +314,11 @@ rad_auth_init(grad_request_t *radreq, int activefd)
          * See if the user has access to this huntgroup.
          */
         if (!huntgroup_access(radreq, &loc)) {
-                grad_log_req(L_NOTICE, radreq,
+                grad_log_req(L_NOTICE, radreq->request,
 			     _("Access denied by huntgroup %s:%d"),
 			     loc.file, loc.line);
                 radius_send_reply(RT_ACCESS_REJECT, radreq,
-                                  radreq->request, NULL, activefd);
+                                  radreq->request->avlist, NULL, activefd);
                 return -1;
         }
 
@@ -271,14 +357,14 @@ enum list_id {
 };
 
 typedef struct auth_mach {
-        grad_request_t *req;
+        radiusd_request_t *req;
         grad_avp_t *user_check;
         grad_avp_t *user_reply;
         int        activefd;
         
         grad_avp_t *namepair;
         grad_avp_t *check_pair;
-        char       userpass[AUTH_STRING_LEN+1];
+        char       userpass[GRAD_STRING_LENGTH+1];
 
         char       *user_msg;
         struct obstack msg_stack;
@@ -371,7 +457,7 @@ auth_log(AUTH_MACH *m, const char *diag, const char *pass,
 	 const char *reason, const char *addstr)
 {
         if (reason)
-                grad_log_req(L_NOTICE, m->req,
+                grad_log_req(L_NOTICE, m->req->request,
 			     "%s [%s%s%s]: %s%s",
 			     diag,
 			     m->namepair->avp_strvalue,
@@ -380,7 +466,7 @@ auth_log(AUTH_MACH *m, const char *diag, const char *pass,
 			     reason,
 			     addstr ? addstr : "");
         else
-                grad_log_req(L_NOTICE, m->req,
+                grad_log_req(L_NOTICE, m->req->request,
 			     "%s [%s%s%s]",
 			     diag,
 			     m->namepair->avp_strvalue,
@@ -400,7 +486,7 @@ is_log_mode(AUTH_MACH *m, int mask)
              p;
              p = p->next ? grad_avl_find(p->next, DA_LOG_MODE_MASK) : NULL)
                 xmask |= p->avp_lvalue;
-        for (p = grad_avl_find(m->req->request, DA_LOG_MODE_MASK);
+        for (p = grad_avl_find(m->req->request->avlist, DA_LOG_MODE_MASK);
              p;
              p = p->next ? grad_avl_find(p->next, DA_LOG_MODE_MASK) : NULL)
                 xmask |= p->avp_lvalue;
@@ -422,23 +508,23 @@ auth_finish_msg(AUTH_MACH *m)
                 obstack_grow(&m->msg_stack, m->user_msg, strlen(m->user_msg));
         obstack_1grow(&m->msg_stack, 0);
         return radius_xlate(&m->msg_stack, obstack_finish(&m->msg_stack),
-                            m->req, m->user_reply);
+                            m->req->request, m->user_reply);
 }
 
 /* Check password. */
 static enum auth_status 
-rad_check_password(grad_request_t *radreq, AUTH_MACH *m, time_t *exp)
+rad_check_password(radiusd_request_t *radreq, AUTH_MACH *m, time_t *exp)
 {
         char *ptr;
         char *real_password = NULL;
-        char name[AUTH_STRING_LEN];
+        char name[GRAD_STRING_LENGTH];
         grad_avp_t *auth_item;
         grad_avp_t *tmp;
         int auth_type = -1;
         int length;
         enum auth_status result = auth_ok;
         char *authdata = NULL;
-        char pw_digest[AUTH_VECTOR_LEN];
+        char pw_digest[GRAD_AUTHENTICATOR_LENGTH];
         int pwlen;
         char *pwbuf;
         char *challenge;
@@ -464,17 +550,17 @@ rad_check_password(grad_request_t *radreq, AUTH_MACH *m, time_t *exp)
         /* Find the password sent by the user. If it's not present,
            authentication fails. */
         
-        if (auth_item = grad_avl_find(radreq->request, DA_CHAP_PASSWORD))
+        if (auth_item = grad_avl_find(radreq->request->avlist, DA_CHAP_PASSWORD))
                 auth_type = DV_AUTH_TYPE_LOCAL;
         else
-                auth_item = grad_avl_find(radreq->request, DA_USER_PASSWORD);
+                auth_item = grad_avl_find(radreq->request->avlist, DA_USER_PASSWORD);
         
         /* Decrypt the password. */
         if (auth_item) {
                 if (auth_item->avp_strlength == 0)
                         m->userpass[0] = 0;
                 else
-                        req_decrypt_password(m->userpass, radreq,
+                        req_decrypt_password(m->userpass, radreq->request,
                                              auth_item);
         } else /* if (auth_item == NULL) */
                 return auth_fail;
@@ -536,7 +622,7 @@ rad_check_password(grad_request_t *radreq, AUTH_MACH *m, time_t *exp)
                 if (pam_pass(name, m->userpass, authdata, &m->user_msg) != 0)
                         result = auth_fail;
 #else
-                grad_log_req(L_ERR, radreq,
+                grad_log_req(L_ERR, radreq->request,
                              _("PAM authentication not available"));
                 result = auth_nouser;
 #endif
@@ -590,12 +676,13 @@ rad_check_password(grad_request_t *radreq, AUTH_MACH *m, time_t *exp)
 		   allocate it */
                 length = strlen(real_password);
 		
-                if (tmp = grad_avl_find(radreq->request, DA_CHAP_CHALLENGE)) {
+                if (tmp = grad_avl_find(radreq->request->avlist,
+					DA_CHAP_CHALLENGE)) {
                         challenge = tmp->avp_strvalue;
                         challenge_len = tmp->avp_strlength;
                 } else {
-                        challenge = radreq->vector;
-                        challenge_len = AUTH_VECTOR_LEN;
+                        challenge = radreq->request->authenticator;
+                        challenge_len = GRAD_AUTHENTICATOR_LENGTH;
                 }
 
                 pwlen = 1 + length + challenge_len;
@@ -613,7 +700,7 @@ rad_check_password(grad_request_t *radreq, AUTH_MACH *m, time_t *exp)
 		
                 /* Compare them */
                 if (memcmp(pw_digest, auth_item->avp_strvalue + 1,
-			   CHAP_VALUE_LENGTH) != 0)
+			   GRAD_CHAP_VALUE_LENGTH) != 0)
                         result = auth_fail;
                 else
                         strcpy(m->userpass, real_password);
@@ -654,7 +741,7 @@ radius_check_expiration(AUTH_MACH *m, time_t *exp)
 
 
 int
-rad_authenticate(grad_request_t *radreq, int activefd)
+rad_authenticate(radiusd_request_t *radreq, int activefd)
 {
         enum auth_state oldstate;
         struct auth_state_s *sp;
@@ -669,7 +756,7 @@ rad_authenticate(grad_request_t *radreq, int activefd)
         m.user_msg   = NULL;
         obstack_init(&m.msg_stack);
 
-        m.namepair = grad_avl_find(m.req->request, DA_USER_NAME);
+        m.namepair = grad_avl_find(m.req->request->avlist, DA_USER_NAME);
 
         debug(1, ("auth: %s", m.namepair->avp_strvalue)); 
         m.state = as_init;
@@ -682,7 +769,7 @@ rad_authenticate(grad_request_t *radreq, int activefd)
                         
                         switch (sp->list) {
                         case L_req:
-                                p = m.req->request;
+                                p = m.req->request->avlist;
                                 break;
                         case L_check:
                                 p = m.user_check;
@@ -729,7 +816,7 @@ rad_authenticate(grad_request_t *radreq, int activefd)
 void
 sfn_init(AUTH_MACH *m)
 {
-        grad_request_t *radreq = m->req;
+        radiusd_request_t *radreq = m->req;
         grad_avp_t *pair_ptr;
 
 	switch (radreq->server_code) {
@@ -761,7 +848,7 @@ sfn_init(AUTH_MACH *m)
 	 * If the request is processing a menu, service it here.
 	 */
 	if (radreq->server_code == 0
-	    && (pair_ptr = grad_avl_find(m->req->request, DA_STATE)) != NULL
+	    && (pair_ptr = grad_avl_find(m->req->request->avlist, DA_STATE)) != NULL
 	    && strncmp(pair_ptr->avp_strvalue, "MENU=", 5) == 0) {
 		menu_reply(m->req, m->activefd);
 		newstate(as_stop);
@@ -806,7 +893,7 @@ sfn_scheme(AUTH_MACH *m)
         grad_avp_t *reply = NULL;
         
         if (!use_guile) {
-                grad_log_req(L_ERR, m->req,
+                grad_log_req(L_ERR, m->req->request,
                              _("Guile authentication disabled in config"));
                 newstate(as_reject_cleanup);
                 return;
@@ -847,7 +934,7 @@ auth_failure(AUTH_MACH *m)
 		return;
 
 	obstack_init(&stk);
-	cmd = util_xlate(&m->msg_stack, pair->avp_strvalue, m->req);
+	cmd = util_xlate(&m->msg_stack, pair->avp_strvalue, m->req->request);
 	switch (cmd[0]) {
 	case '(':
 		scheme_eval_boolean_expr(cmd);
@@ -864,7 +951,7 @@ auth_failure(AUTH_MACH *m)
 void
 sfn_validate(AUTH_MACH *m)
 {
-        grad_request_t *radreq = m->req;
+        radiusd_request_t *radreq = m->req;
 	enum auth_status rc;
 	time_t exp;
 	grad_avp_t *pair;
@@ -982,7 +1069,7 @@ sfn_realmuse(AUTH_MACH *m)
 void
 sfn_simuse(AUTH_MACH *m)
 {
-        char  name[AUTH_STRING_LEN];
+        char  name[GRAD_STRING_LENGTH];
         int rc;
         int count;
 	
@@ -999,7 +1086,7 @@ sfn_simuse(AUTH_MACH *m)
                         (m->check_pair->avp_lvalue > 1) ?
                         MSG_MULTIPLE_LOGIN : MSG_SECOND_LOGIN);
 
-        grad_log_req(L_WARN, m->req,
+        grad_log_req(L_WARN, m->req->request,
 		     _("Multiple logins: [%s] max. %ld%s"),
 		     m->namepair->avp_strvalue,
 		     m->check_pair->avp_lvalue,
@@ -1035,7 +1122,7 @@ sfn_time(AUTH_MACH *m)
                  */
                 auth_format_msg(m, MSG_TIMESPAN_VIOLATION);
                 grad_log_req(L_ERR,
-			     m->req,
+			     m->req->request,
 			     _("Outside allowed timespan (%s)"),
 			     m->check_pair->avp_strvalue);
                 newstate(as_reject_cleanup);
@@ -1059,7 +1146,8 @@ sfn_ipaddr(AUTH_MACH *m)
         
         /* Assign an IP if necessary */
         if (!grad_avl_find(m->user_reply, DA_FRAMED_IP_ADDRESS)) {
-                if (p = grad_avl_find(m->req->request, DA_FRAMED_IP_ADDRESS)) {
+                if (p = grad_avl_find(m->req->request->avlist,
+				      DA_FRAMED_IP_ADDRESS)) {
                         /* termserver hint */
                         grad_avl_add_pair(&m->user_reply, grad_avp_dup(p));
                 }
@@ -1231,26 +1319,29 @@ req_decrypt_password(char *password, grad_request_t *req, grad_avp_t *pair)
         char *s;
         
         if (!pair) {
-                pair = grad_avl_find(req->request, DA_USER_PASSWORD);
+                pair = grad_avl_find(req->avlist, DA_USER_PASSWORD);
                 if (!pair)
                         return;
         }
 
-	if (pair->prop & AP_ENCRYPT_RFC2138) {
+	if (pair->prop & GRAD_AP_ENCRYPT_RFC2138) {
 		/* Determine whether we need to use broken decoding */
 		nas = grad_nas_request_to_nas(req);
 		if (nas
 		    && (s = grad_envar_lookup(nas->args, "broken_pass")) != NULL
 		    && s[0] == '1')
 			grad_decrypt_password_broken(password, pair,
-						     req->vector, req->secret);
+						     req->authenticator,
+						     req->secret);
 		else
 			grad_decrypt_password(password, pair,
-					      req->vector, req->secret);
-	} else if (pair->prop & AP_ENCRYPT_RFC2868) {
+					      req->authenticator,
+					      req->secret);
+	} else if (pair->prop & GRAD_AP_ENCRYPT_RFC2868) {
 		u_char tag; /* FIXME: not accessible for user */
 		grad_decrypt_tunnel_password(password, 
 					     &tag, pair,
-					     req->vector, req->secret);
+					     req->authenticator,
+					     req->secret);
 	}
 }
