@@ -1,6 +1,8 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2002,2003 Sergey Poznyakoff
-  
+   Copyright (C) 2002,2003,2004 Free Software Foundation, Inc.
+
+   Written by Sergey Poznyakoff
+
    GNU Radius is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -15,8 +17,6 @@
    along with GNU Radius; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define RADIUS_MODULE_EXEC_C
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -38,86 +38,239 @@
 #include <grp.h>
 #include <string.h>
 #include <syslog.h>
+
 #include <radiusd.h>
-#include <obstack1.h>
-#include <argcv.h>
-#include <symtab.h>
+#include <radius/argcv.h>
+#include <radius/symtab.h>
 #include <rewrite.h>
 
 int
-radius_change_uid(struct passwd *pwd)
+radius_get_user_ids(RADIUS_USER *usr, const char *name)
 {
-	if (pwd->pw_gid != 0 && setgid(pwd->pw_gid)) {
-		radlog(L_ERR|L_PERROR,
-		       _("setgid(%d) failed"), pwd->pw_gid);
-		return -1;
-	}
-	if (pwd->pw_uid != 0) {
-#if defined(HAVE_SETEUID)
-		if (seteuid(pwd->pw_uid)) {
-			radlog(L_ERR|L_PERROR,
-			       _("seteuid(%d) failed (ruid=%d, euid=%d)"),
-			       pwd->pw_uid, getuid(), geteuid());
-			return -1;
-		}
-#elif defined(HAVE_SETREUID)
-		if (setreuid(0, pwd->pw_uid)) {
-			radlog(L_ERR|L_PERROR,
-			       _("setreuid(0,%d) failed (ruid=%d, euid=%d)"),
-			       pwd->pw_uid, getuid(), geteuid());
-			return -1;
-		}
-#else
-# error "*** NO WAY TO SET EFFECTIVE UID IN radius_change_uid() ***"
-#endif
+	struct passwd *pwd = getpwnam(name);
+
+	if (!pwd) {
+		grad_log(L_ERR, _("no such user: %s"), name);
+		return 1;
 	}
+	grad_string_replace(&usr->username, name);
+	usr->uid = pwd->pw_uid;
+	usr->gid = pwd->pw_gid;
 	return 0;
 }
 
+int
+radius_switch_to_user(RADIUS_USER *usr)
+{
+	int rc = 0;
+	gid_t emptygidset[1];
+
+	if (usr->username == NULL)
+		return 0;
+	
+	/* Reset group permissions */
+	emptygidset[0] = usr->gid ? usr->gid : getegid();
+	if (geteuid() == 0 && setgroups(1, emptygidset)) {
+		grad_log(L_ERR|L_PERROR,
+		         _("setgroups(1, %lu) failed"),
+		         (u_long) emptygidset[0]);
+		rc = 1;
+	}
+
+	/* Switch to the user's gid. On some OSes the effective gid must
+	   be reset first */
+
+#if defined(HAVE_SETEGID)
+	if ((rc = setegid(usr->gid)) < 0)
+		grad_log(L_ERR|L_PERROR,
+		         _("setegid(%lu) failed"), (u_long) usr->gid);
+#elif defined(HAVE_SETREGID)
+	if ((rc = setregid(usr->gid, usr->gid)) < 0)
+		grad_log(L_ERR|L_PERROR,
+		         _("setregid(%lu,%lu) failed"),
+		         (u_long) usr->gid, (u_long) usr->gid);
+#elif defined(HAVE_SETRESGID)
+	if ((rc = setresgid(usr->gid, usr->gid, usr->gid)) < 0)
+		grad_log(L_ERR|L_PERROR,
+		         _("setresgid(%lu,%lu,%lu) failed"),
+		         (u_long) usr->gid,
+		         (u_long) usr->gid,
+		         (u_long) usr->gid);
+#endif
+
+	if (rc == 0 && usr->gid != 0) {
+		if ((rc = setgid(usr->gid)) < 0 && getegid() != usr->gid) 
+			grad_log(L_ERR|L_PERROR,
+			         _("setgid(%lu) failed"), (u_long) usr->gid);
+		if (rc == 0 && getegid() != usr->gid) {
+			grad_log(L_ERR,
+			         _("cannot set effective gid to %lu"),
+			         (u_long) usr->gid);
+			rc = 1;
+		}
+	}
+
+	/* Now reset uid */
+	if (rc == 0 && usr->uid != 0) {
+		uid_t euid;
+
+		if (setuid(usr->uid)
+		    || geteuid() != usr->uid
+		    || (getuid() != usr->uid
+			&& (geteuid() == 0 || getuid() == 0))) {
+			
+#if defined(HAVE_SETREUID)
+			if (geteuid() != usr->uid) {
+				if (setreuid(usr->uid, -1) < 0) { 
+					grad_log(L_ERR|L_PERROR,
+					         _("setreuid(%lu,-1) failed"),
+					         (u_long) usr->uid);
+					rc = 1;
+				}
+				if (setuid(usr->uid) < 0) {
+					grad_log(L_ERR|L_PERROR,
+					         _("second setuid(%lu) failed"),
+					         (u_long) usr->uid);
+					rc = 1;
+				}
+			} else
+#endif
+				{
+					grad_log(L_ERR|L_PERROR,
+					         _("setuid(%lu) failed"),
+					         (u_long) usr->uid);
+					rc = 1;
+				}
+		}
+	
+
+		euid = geteuid();
+		if (usr->uid != 0 && setuid(0) == 0) {
+			grad_log(L_ERR, 
+                                 _("seteuid(0) succeeded when it should not"));
+			rc = 1;
+		} else if (usr->uid != euid && setuid(euid) == 0) {
+			grad_log(L_ERR, 
+                                 _("cannot drop non-root setuid privileges"));
+			rc = 1;
+		}
+
+	}
+
+	return rc;
+}
+
+int
+radius_exec_command(char *cmd)
+{
+        int n;
+        grad_avp_t *vp;
+	pid_t pid;
+	int status;
+	RETSIGTYPE (*oldsig)();
+	
+        if (cmd[0] != '/') {
+                grad_log(L_ERR,
+   _("radius_exec_program(): won't execute, not an absolute pathname: %s"),
+                         cmd);
+                return -1;
+        }
+
+	if ((oldsig = grad_set_signal(SIGCHLD, SIG_DFL)) == SIG_ERR) {
+		grad_log(L_ERR|L_PERROR, _("can't reset SIGCHLD"));
+		return -1;
+        } 
+
+        if ((pid = fork()) == 0) {
+                int argc;
+                char **argv;
+
+                argcv_get(cmd, "", NULL, &argc, &argv);
+                
+                for (n = grad_max_fd(); n >= 3; n--)
+                        close(n);
+
+                chdir("/tmp");
+
+                if (radius_switch_to_user(&exec_user))
+			exit(2);
+		
+                execvp(argv[0], argv);
+
+                /* Report error via syslog: we might not be able
+		   to restore initial privileges if we were started
+		   as non-root. */
+                openlog("radiusd", LOG_PID, LOG_USER);
+                syslog(LOG_ERR, "can't run %s (ruid=%lu, euid=%lu): %m",
+                       argv[0], (u_long) getuid(), (u_long) geteuid());
+                exit(2);
+        }
+
+        /* Parent branch */ 
+        if (pid < 0) {
+                grad_log(L_ERR|L_PERROR, "fork");
+                return -1;
+        }
+
+	waitpid(pid, &status, 0);
+	if (grad_set_signal(SIGCHLD, oldsig) == SIG_ERR)
+		grad_log(L_CRIT|L_PERROR,
+			 _("can't restore SIGCHLD"));
+
+        if (WIFEXITED(status)) {
+                status = WEXITSTATUS(status);
+                debug(1, ("returned: %d", status));
+                if (status == 2) {
+                        grad_log(L_ERR,
+                                 _("can't run external program `%s' "
+                                   "(reason reported via syslog channel "
+                                   "user.err)"),
+			         cmd);
+                }
+        } else {
+		char buffer[RAD_BUFFER_SIZE];
+		
+		format_exit_status(buffer, sizeof buffer, status);
+		
+		grad_log(L_ERR,
+		         _("external program `%s' %s"), cmd, buffer);
+	}
+	
+        return status;
+}
+
 /* Execute a program on successful authentication.
    Return 0 if exec_wait == 0.
    Return the exit code of the called program if exec_wait != 0. */
 int
-radius_exec_program(char *cmd, RADIUS_REQ *req, VALUE_PAIR **reply,
+radius_exec_program(char *cmd, radiusd_request_t *req, grad_avp_t **reply,
 		    int exec_wait)
 {
         int p[2];
         int n;
         char *ptr, *errp;
-        VALUE_PAIR *vp;
+        grad_avp_t *vp;
         FILE *fp;
         int line_num;
         char buffer[RAD_BUFFER_SIZE];
-        struct passwd *pwd;
 	pid_t pid;
 	int status;
 	RETSIGTYPE (*oldsig)();
 	
         if (cmd[0] != '/') {
-                radlog(L_ERR,
+                grad_log(L_ERR,
    _("radius_exec_program(): won't execute, not an absolute pathname: %s"),
-                       cmd);
-                return -1;
-        }
-
-        /* Check user/group
-           FIXME: This should be checked *once* after re-reading the
-           configuration */
-        pwd = getpwnam(exec_user);
-        if (!pwd) {
-                radlog(L_ERR,
-    _("radius_exec_program(): won't execute, no such user: %s"),
-                       exec_user);
+                         cmd);
                 return -1;
         }
 
         if (exec_wait) {
                 if (pipe(p) != 0) {
-                        radlog(L_ERR|L_PERROR, _("couldn't open pipe"));
+                        grad_log(L_ERR|L_PERROR, _("couldn't open pipe"));
                         return -1;
                 }
-		if ((oldsig = signal(SIGCHLD, SIG_DFL)) == SIG_ERR) {
-			radlog(L_ERR|L_PERROR, _("can't reset SIGCHLD"));
+		if ((oldsig = grad_set_signal(SIGCHLD, SIG_DFL)) == SIG_ERR) {
+			grad_log(L_ERR|L_PERROR, _("can't reset SIGCHLD"));
 			return -1;
 		}
         } 
@@ -130,25 +283,27 @@ radius_exec_program(char *cmd, RADIUS_REQ *req, VALUE_PAIR **reply,
                 obstack_init(&s);
                 
                 /* child branch */
-                ptr = radius_xlate(&s, cmd, req, reply ? *reply : NULL);
+                ptr = radius_xlate(&s, cmd, req->request,
+				   reply ? *reply : NULL);
 
                 debug(1, ("command line: %s", ptr));
 
-                argcv_get(ptr, "", &argc, &argv);
+                argcv_get(ptr, "", NULL, &argc, &argv);
                 
                 if (exec_wait) {
                         if (close(p[0]))
-                                radlog(L_ERR|L_PERROR, _("can't close pipe"));
-                        if (dup2(p[1], 1) != 1)
-                                radlog(L_ERR|L_PERROR, _("can't dup stdout"));
-                }
+                                grad_log(L_ERR|L_PERROR, _("can't close pipe"));
+                        if (p[1] != 1 && dup2(p[1], 1) != 1)
+                                grad_log(L_ERR|L_PERROR, _("can't dup stdout"));
+                } else
+			close(1);
 
-                for (n = getmaxfd(); n >= 3; n--)
+                for (n = grad_max_fd(); n >= 3; n--)
                         close(n);
 
                 chdir("/tmp");
 
-                if (radius_change_uid(pwd))
+                if (radius_switch_to_user(&exec_user))
 			exit(2);
 		
                 execvp(argv[0], argv);
@@ -157,21 +312,21 @@ radius_exec_program(char *cmd, RADIUS_REQ *req, VALUE_PAIR **reply,
 		   to restore initial privileges if we were started
 		   as non-root. */
                 openlog("radiusd", LOG_PID, LOG_USER);
-                syslog(LOG_ERR, "can't run %s (ruid=%d, euid=%d): %m",
-                       argv[0], getuid(), geteuid());
+                syslog(LOG_ERR, "can't run %s (ruid=%lu, euid=%lu): %m",
+                       argv[0], (u_long) getuid(), (u_long) geteuid());
                 exit(2);
         }
 
         /* Parent branch */ 
         if (pid < 0) {
-                radlog(L_ERR|L_PERROR, "fork");
+                grad_log(L_ERR|L_PERROR, "fork");
                 return -1;
         }
         if (!exec_wait)
                 return 0;
 
         if (close(p[1]))
-                radlog(L_ERR|L_PERROR, _("can't close pipe"));
+                grad_log(L_ERR|L_PERROR, _("can't close pipe"));
 
         fp = fdopen(p[0], "r");
 
@@ -182,42 +337,43 @@ radius_exec_program(char *cmd, RADIUS_REQ *req, VALUE_PAIR **reply,
                 line_num++;
                 debug(1, ("got `%s'", buffer));
                 if (userparse(ptr, &vp, &errp)) {
-                        radlog(L_ERR,
-			       _("<stdout of %s>:%d: %s"),
-			       cmd, line_num, errp);
-                        avl_free(vp);
+                        grad_log(L_ERR,
+			         _("<stdout of %s>:%d: %s"),
+			         cmd, line_num, errp);
+                        grad_avl_free(vp);
                         vp = NULL;
                 }
         }
 
         fclose(fp);
 
-        if (vp) {
-                avl_merge(reply, &vp);
-		avl_free(vp);
-	}
-
 	waitpid(pid, &status, 0);
-	if (signal(SIGCHLD, oldsig) == SIG_ERR)
-		radlog(L_CRIT|L_PERROR,
-			_("can't restore SIGCHLD"));
+	if (grad_set_signal(SIGCHLD, oldsig) == SIG_ERR)
+		grad_log(L_CRIT|L_PERROR,
+			 _("can't restore SIGCHLD"));
 
         if (WIFEXITED(status)) {
                 status = WEXITSTATUS(status);
                 debug(1, ("returned: %d", status));
                 if (status == 2) {
-                        radlog(L_ERR,
-                               _("can't run external program `%s' (reason reported via syslog channel user.err)"),
-			       cmd);
+                        grad_log(L_ERR,
+                                 _("can't run external program `%s' "
+                                   "(reason reported via syslog channel "
+                                   "user.err)"),
+			         cmd);
                 }
-                return status;
         } else {
 		format_exit_status(buffer, sizeof buffer, status);
-
-		radlog(L_ERR,
-		       _("external program `%s' %s"), cmd, buffer);
+		
+		grad_log(L_ERR,
+		         _("external program `%s' %s"), cmd, buffer);
 	}
-        return 1;
+
+        if (vp && reply) 
+                grad_avl_merge(reply, &vp);
+	grad_avl_free(vp);
+
+        return status;
 }
 
 pid_t
@@ -225,17 +381,8 @@ radius_run_filter(int argc, char **argv, char *errfile, int *p)
 {
 	pid_t  pid;
 	int    rightp[2], leftp[2];
-	struct passwd *pwd;
 	int i;
 
-        pwd = getpwnam(exec_user);
-        if (!pwd) {
-                radlog(L_ERR,
-		       _("won't execute, no such user: %s"),
-                       exec_user);
-                return -1;
-        }
-
 	pipe(leftp);
 	pipe(rightp);
     
@@ -245,28 +392,33 @@ radius_run_filter(int argc, char **argv, char *errfile, int *p)
 	case 0:
 		/* attach the pipes */
 
+		/* Left-end. It is important that it be processed first,
+		   since it may contain descriptors 0 and 1 */
+		if (leftp[0] != 0) {
+			close(0);
+			dup2(leftp[0], 0);
+		}
+		close(leftp[1]);
+		
 		/* Right-end */
-		close(1);
-		dup2(rightp[1], 1);
+		if (rightp[1] != 1) {
+			close(1);
+			dup2(rightp[1], 1);
+		}
 		close(rightp[0]); 
 
-		/* Left-end */
-		close(0);
-		dup2(leftp[0], 0);
-		close(leftp[1]);
-
 		/* Error output */
-		i = open(errfile, O_CREAT|O_WRONLY, 0644);
+		i = open(errfile, O_CREAT|O_WRONLY|O_APPEND, 0644);
                 if (i > 0 && i != 2) {
                         dup2(i, 2);
 			close(i);
 		}
 		
 		/* Close unneded descripitors */
-		for (i = getmaxfd(); i > 2; i--)
+		for (i = grad_max_fd(); i > 2; i--)
 			close(i);
 
-		if (radius_change_uid(pwd))
+		if (radius_switch_to_user(&exec_user))
 			exit(2);
 		execvp(argv[0], argv);
 		
@@ -329,7 +481,7 @@ struct filter_symbol {
 	int    output;              /* Output file descriptor */
 }; 
 
-static Symtab *filter_tab;
+static grad_symtab_t *filter_tab;
 
 struct cleanup_info {
 	pid_t pid;
@@ -337,19 +489,20 @@ struct cleanup_info {
 };
 	
 static int
-filter_cleanup_proc(void *ptr, Filter *filter)
+filter_cleanup_proc(void *ptr, grad_symbol_t *sym)
 {
 	struct cleanup_info *info = ptr;
+	Filter *filter = (Filter *) sym;
 
 	if (filter->pid == info->pid) {
 		static char buffer[512];
 		
 		format_exit_status(buffer, sizeof buffer, info->status);
-		radlog(L_ERR,
-		       _("filter %s (pid %d) %s (in: %u, out: %u)"),
-		       filter->name, filter->pid,
-		       buffer,
-		       filter->lines_input, filter->lines_output);
+		grad_log(L_ERR,
+		         _("filter %s (pid %d) %s (in: %u, out: %u)"),
+		         filter->name, filter->pid,
+		         buffer,
+		         filter->lines_input, filter->lines_output);
 		filter->pid = 0;
 		return 1;
 	}
@@ -362,7 +515,7 @@ filter_cleanup(pid_t pid, int status)
 	struct cleanup_info info;
 	info.pid = pid;
 	info.status = status;
-	symtab_iterate(filter_tab, filter_cleanup_proc, &info);
+	grad_symtab_iterate(filter_tab, filter_cleanup_proc, &info);
 }
 
 void
@@ -394,13 +547,13 @@ filter_kill(Filter *filter)
 }
 
 static Filter *
-filter_open(char *name, RADIUS_REQ *req, int type, int *errp)
+filter_open(char *name, radiusd_request_t *req, int type, int *errp)
 {
-	Filter *filter = sym_lookup(filter_tab, name);
+	Filter *filter = grad_sym_lookup(filter_tab, name);
 	if (!filter) {
-		radlog_req(L_ERR, req,
-			   _("filter %s is not declared"),
-			   name);
+		grad_log_req(L_ERR, req->request,
+			     _("filter %s is not declared"),
+			     name);
 		*errp = -1;
 		return NULL;
 	}
@@ -415,9 +568,9 @@ filter_open(char *name, RADIUS_REQ *req, int type, int *errp)
 						pipe);
 		
 		if (filter->pid <= 0) {
-			radlog_req(L_ERR|L_PERROR, req,
-				   _("cannot run filter %s"),
-				   name);
+			grad_log_req(L_ERR|L_PERROR, req->request,
+				     _("cannot run filter %s"),
+				     name);
 			filter = NULL;
 		} else { 
 			if (!filter->descr[R_AUTH].wait_reply
@@ -433,7 +586,8 @@ filter_open(char *name, RADIUS_REQ *req, int type, int *errp)
 	}
 
 	if (filter && kill(filter->pid, 0)) {
-		radlog_req(L_ERR|L_PERROR, req, _("filter %s"), name);
+		grad_log_req(L_ERR|L_PERROR, req->request,
+			     _("filter %s"), name);
 		filter_close(filter);
 		filter = NULL;
 	}
@@ -441,33 +595,14 @@ filter_open(char *name, RADIUS_REQ *req, int type, int *errp)
 	return filter;
 }
 
-static char *
-filter_xlate(struct obstack *sp, char *fmt, RADIUS_REQ *radreq)
+char *
+filter_xlate(struct obstack *sp, char *fmt, radiusd_request_t *radreq)
 {
-	char *str;
-	
-	if (fmt[0] == '=') {
-		Datatype type;
-		Datum datum;
-		
-		if (interpret(fmt+1, radreq, &type, &datum)) 
-			return NULL;
-		if (type != String) {
-			radlog(L_ERR, "%s: %s",
-			       fmt+1, _("wrong return type"));
-			return NULL;
-		}
-		obstack_grow(sp, datum.sval, strlen(datum.sval)+1);
-		efree(datum.sval);
-		str = obstack_finish(sp);
-	} else {
-		str = radius_xlate(sp, fmt, radreq, NULL);
-	}
-	return str;
+	return util_xlate(sp, fmt, radreq->request);
 }
 
 static int
-filter_write(Filter *filter, char *fmt, RADIUS_REQ *radreq)
+filter_write(Filter *filter, char *fmt, radiusd_request_t *radreq)
 {
 	int rc, length;
 	struct obstack stack;
@@ -532,7 +667,7 @@ filter_read(Filter *filter, int type, char *buffer, size_t buflen)
    Returns: 0   -- Authentication succeeded
             !0  -- Authentication failed */
 int
-filter_auth(char *name, RADIUS_REQ *req, VALUE_PAIR **reply_pairs)
+filter_auth(char *name, radiusd_request_t *req, grad_avp_t **reply_pairs)
 {
 	Filter *filter;
 	int rc = -1;
@@ -554,31 +689,31 @@ filter_auth(char *name, RADIUS_REQ *req, VALUE_PAIR **reply_pairs)
 		status = filter_read(filter, R_AUTH, buffer, sizeof buffer);
 			
 		if (status <= 0) {
-			radlog(L_ERR|L_PERROR,
-		       	       _("reading from filter %s"),
-		       	       filter->name);
+			grad_log(L_ERR|L_PERROR,
+		       	         _("reading from filter %s"),
+		       	         filter->name);
 			filter_close(filter);
 			rc = err;
 		} else if (isdigit(buffer[0])) {
 			char *ptr;
-			VALUE_PAIR *vp = NULL;
+			grad_avp_t *vp = NULL;
 			char *errp;
 
 			debug(1, ("%s > \"%s\"", filter->name, buffer));
 			rc = strtoul(buffer, &ptr, 0);
 			if (userparse(ptr, &vp, &errp)) {
-				radlog(L_ERR,
-				       _("<stdout of %s>:%d: %s"),
-				       filter->name,
-				       filter->lines_output,
-				       errp);
-				avl_free(vp);
-			} else
-				avl_merge(reply_pairs, &vp);
+				grad_log(L_ERR,
+				         _("<stdout of %s>:%d: %s"),
+				         filter->name,
+				         filter->lines_output,
+				         errp);
+				grad_avl_free(vp);
+			} else 
+				grad_avl_merge(reply_pairs, &vp);
 		} else {
-			radlog(L_ERR,
-			       _("filter %s (auth): bad output: %s"),
-			       filter->name, buffer);
+			grad_log(L_ERR,
+			         _("filter %s (auth): bad output: %s"),
+			         filter->name, buffer);
 			rc = err;
 		}
 	}
@@ -587,7 +722,7 @@ filter_auth(char *name, RADIUS_REQ *req, VALUE_PAIR **reply_pairs)
 }
 
 int
-filter_acct(char *name, RADIUS_REQ *req)
+filter_acct(char *name, radiusd_request_t *req)
 {
 	Filter *filter;
 	int rc = -1;
@@ -609,9 +744,9 @@ filter_acct(char *name, RADIUS_REQ *req)
 		status = filter_read(filter, R_ACCT, buffer, sizeof buffer);
 
 		if (status <= 0) {
-			radlog(L_ERR|L_PERROR,
-		       	       _("reading from filter %s"),
-		       	       filter->name);
+			grad_log(L_ERR|L_PERROR,
+		       	         _("reading from filter %s"),
+		       	         filter->name);
 			rc = err;
 			filter_close(filter);
 		} else if (isdigit(buffer[0])) {
@@ -620,15 +755,15 @@ filter_acct(char *name, RADIUS_REQ *req)
 			debug(1, ("%s > \"%s\"", filter->name, buffer));
 			rc = strtoul(buffer, &ptr, 0);
 			if (!isspace(*ptr)) {
-				radlog(L_ERR,
-				       _("filter %s (acct): bad output: %s"),
-				       filter->name, buffer);
+				grad_log(L_ERR,
+				         _("filter %s (acct): bad output: %s"),
+				         filter->name, buffer);
 				return -1;
 			}
 		} else {
-			radlog(L_ERR,
-			       _("filter %s (acct): bad output: %s"),
-			       filter->name, buffer);
+			grad_log(L_ERR,
+			         _("filter %s (acct): bad output: %s"),
+			         filter->name, buffer);
 			rc = err;
 		}
 	}
@@ -644,10 +779,10 @@ static struct filter_symbol filter_symbol;
 static int
 free_symbol_entry(Filter *filter)
 {
-	efree(filter->descr[R_AUTH].input_fmt);
-	efree(filter->descr[R_ACCT].input_fmt);
+	grad_free(filter->descr[R_AUTH].input_fmt);
+	grad_free(filter->descr[R_ACCT].input_fmt);
 	argcv_free(filter->argc, filter->argv);
-	efree(filter->errfile);
+	grad_free(filter->errfile);
 	if (filter->pid > 0)
 		filter_close(filter);
 	return 0;
@@ -658,10 +793,10 @@ filters_stmt_term(int finish, void *block_data, void *handler_data)
 {
 	if (!finish) {
 		if (filter_tab)
-			symtab_clear(filter_tab);
+			grad_symtab_clear(filter_tab);
 		else
-			filter_tab = symtab_create(sizeof(Filter),
-						   free_symbol_entry);
+			filter_tab = grad_symtab_create(sizeof(Filter),
+						        free_symbol_entry);
 	}
 	return 0;
 }
@@ -693,14 +828,14 @@ static int
 filter_stmt_end(void *block_data, void *handler_data)
 {
 	if (filter_symbol.argc) {
-		Filter *sym = sym_lookup_or_install(filter_tab,
-						    filter_symbol.name,
-						    1);
+		Filter *sym = grad_sym_lookup_or_install(filter_tab,
+						         filter_symbol.name,
+						         1);
 		if (sym->argc) {
-			radlog(L_ERR,
-			       _("%s:%d: filter already declared at %s:%d"),
-			       cfg_filename, cfg_line_num,
-			       cfg_filename, sym->line_num);
+			grad_log(L_ERR,
+			         _("%s:%d: filter already declared at %s:%d"),
+			         cfg_filename, cfg_line_num,
+			         cfg_filename, sym->line_num);
 			return 0;
 		}
 
@@ -708,9 +843,9 @@ filter_stmt_end(void *block_data, void *handler_data)
 		sym->argc     = filter_symbol.argc;
 		sym->argv     = filter_symbol.argv;
 		sym->descr[R_AUTH].input_fmt =
-			estrdup(filter_symbol.descr[R_AUTH].input_fmt);
+			grad_estrdup(filter_symbol.descr[R_AUTH].input_fmt);
 		sym->descr[R_ACCT].input_fmt =
-			estrdup(filter_symbol.descr[R_ACCT].input_fmt);
+			grad_estrdup(filter_symbol.descr[R_ACCT].input_fmt);
 		sym->descr[R_AUTH].wait_reply =
 			filter_symbol.descr[R_AUTH].wait_reply;
 		sym->descr[R_ACCT].wait_reply =
@@ -719,7 +854,7 @@ filter_stmt_end(void *block_data, void *handler_data)
 			!filter_symbol.descr[R_AUTH].on_fail;
 		sym->descr[R_ACCT].on_fail =
 			!filter_symbol.descr[R_ACCT].on_fail;
-		sym->errfile  = estrdup(filter_symbol.errfile);
+		sym->errfile  = grad_estrdup(filter_symbol.errfile);
 		sym->pid = 0;
 	}
 	return 0;
@@ -739,7 +874,7 @@ exec_path_handler(int argc, cfg_value_t *argv,
 		return 0;
 	}
 	
-	if (argcv_get(argv[1].v.string, "",
+	if (argcv_get(argv[1].v.string, "", NULL,
 		      &filter_symbol.argc, &filter_symbol.argv)) {
 		argcv_free(filter_symbol.argc, filter_symbol.argv);
 		filter_symbol.argc = 0;
@@ -766,10 +901,10 @@ error_log_handler(int argc, cfg_value_t *argv,
 	else if (argv[1].v.string[0] == '/')
 		filter_symbol.errfile = argv[1].v.string;
 	else {
-		char *p = mkfilename(radlog_dir, argv[1].v.string);
+		char *p = grad_mkfilename(radlog_dir, argv[1].v.string);
 		filter_symbol.errfile = cfg_malloc(strlen(p)+1, NULL);
 		strcpy(filter_symbol.errfile, p);
-		efree(p);
+		grad_free(p);
 	}
 	return 0;
 }
