@@ -45,6 +45,7 @@
 #include <radiusd.h>
 #include <checkrad.h>
 #include <radius/raddbm.h>
+#include <rewrite.h>
 
 typedef struct locus_name {
 	struct locus_name *next;
@@ -86,7 +87,7 @@ static grad_list_t /* of grad_matching_rule_t */  *hints;        /* raddb/hints
 static grad_list_t /* of CLIENT */ *clients; /* raddb/clients */
 static grad_list_t /* of RADCK_TYPE */ *radck_type;   /* raddb/nastypes */
 
-static struct keyword op_tab[] = {
+static grad_keyword_t op_tab[] = {
         { "=", grad_operator_equal },
         { "!=", grad_operator_not_equal },
         { ">", grad_operator_greater_than },
@@ -96,21 +97,21 @@ static struct keyword op_tab[] = {
         { 0 }
 };
 
-int paircmp(grad_request_t *req, grad_avp_t *check, char *pusername);
+int paircmp(radiusd_request_t *req, grad_avp_t *check, char *pusername);
 int fallthrough(grad_avp_t *vp);
 /*
  * Static declarations
  */
 static int portcmp(grad_avp_t *check, grad_avp_t *request);
-static int groupcmp(grad_request_t *req, char *groupname, char *username);
+static int groupcmp(radiusd_request_t *req, char *groupname, char *username);
 static int uidcmp(grad_avp_t *check, char *username);
 static void matchrule_free(grad_matching_rule_t **pl);
-static int matches(grad_request_t *req, char *name, grad_matching_rule_t *pl, char *matchpart);
-static int huntgroup_match(grad_request_t *req, char *huntgroup);
-static int user_find_sym(char *name, grad_request_t *req, 
+static int matches(radiusd_request_t *req, char *name, grad_matching_rule_t *pl, char *matchpart);
+static int huntgroup_match(radiusd_request_t *req, char *huntgroup);
+static int user_find_sym(char *name, radiusd_request_t *req, 
                          grad_avp_t **check_pairs, grad_avp_t **reply_pairs);
 #ifdef USE_DBM
-int user_find_db(char *name, grad_request_t *req,
+int user_find_db(char *name, radiusd_request_t *req,
                         grad_avp_t **check_pairs, grad_avp_t **reply_pairs);
 #endif
 static grad_list_t *file_read(int cf_file, char *name);
@@ -176,8 +177,8 @@ add_user_entry(void *closure, grad_locus_t *loc,
                 name = "BEGIN";
 
         if ((check == NULL && reply == NULL)
-            || fix_check_pairs(CF_USERS, loc, name, &check)
-            || fix_reply_pairs(CF_USERS, loc, name, &reply)) {
+            || fix_check_pairs(GRAD_CF_USERS, loc, name, &check)
+            || fix_reply_pairs(GRAD_CF_USERS, loc, name, &reply)) {
                 grad_log_loc(L_ERR, loc,
 			     _("discarding user `%s'"),
 			     name);
@@ -306,14 +307,14 @@ user_next(USER_LOOKUP *lptr)
 }
 
 
-static int match_user(User_symbol *sym, grad_request_t *req,
+static int match_user(User_symbol *sym, radiusd_request_t *req,
                       grad_avp_t **check_pairs, grad_avp_t **reply_pairs);
 
 /*
  * Find matching profile in the hash table
  */
 int
-user_find_sym(char *name, grad_request_t *req,
+user_find_sym(char *name, radiusd_request_t *req,
               grad_avp_t **check_pairs, grad_avp_t **reply_pairs)
 {
         int found = 0;
@@ -323,19 +324,22 @@ user_find_sym(char *name, grad_request_t *req,
         debug(1,("looking for %s", name));
         for (sym = user_lookup(name, &lu); sym; sym = user_next(&lu)) {
                 if (match_user(sym, req, check_pairs, reply_pairs)) {
-                        found = 1;
+			if (lu.state != LU_begin)
+				found = 1;
                         if (!fallthrough(sym->reply))
                                 break;
                         debug(1, ("fall through"));
                         lu.sym = NULL; /* force jump to next state */
                 }
         }
+	if (found)
+		radiusd_sql_reply_attr_query(req, reply_pairs);
         debug(1, ("returning %d", found));
         return found;
 }
 
 int
-match_user(User_symbol *sym, grad_request_t *req,
+match_user(User_symbol *sym, radiusd_request_t *req,
            grad_avp_t **check_pairs, grad_avp_t **reply_pairs)
 {
         grad_avp_t *p;
@@ -376,7 +380,6 @@ match_user(User_symbol *sym, grad_request_t *req,
                 reply_tmp = grad_avl_dup(sym->reply);
                 grad_avl_merge(reply_pairs, &reply_tmp);
                 grad_avl_merge(check_pairs, &check_tmp);
-                radiusd_sql_reply_attr_query(req, reply_pairs);
 
                 grad_avl_free(reply_tmp);
                 grad_avl_free(check_tmp);
@@ -404,7 +407,7 @@ match_user(User_symbol *sym, grad_request_t *req,
  * is done by the caller. user_find() only compares attributes.
  */
 int
-user_find(char *name, grad_request_t *req,
+user_find(char *name, radiusd_request_t *req,
           grad_avp_t **check_pairs, grad_avp_t **reply_pairs)
 {
         int found = 0;
@@ -599,12 +602,13 @@ userparse(char *buffer, grad_avp_t **first_pair, char **errmsg)
 }
 
 static void
-avl_eval_rewrite (grad_request_t *req, grad_matching_rule_t *rule,
+avl_eval_rewrite (radiusd_request_t *req, grad_matching_rule_t *rule,
 		  grad_avp_t *p)
 {
 	for (; p; p = p->next) {
 		if (p->attribute == DA_REWRITE_FUNCTION
-		    && (rewrite_eval(p->avp_strvalue, req, NULL, NULL))) {
+		    && (rewrite_eval(p->avp_strvalue, req->request,
+				     NULL, NULL))) {
 			grad_log_loc(L_ERR, &rule->loc, "%s(): %s",
 				     p->avp_strvalue,
 				     _("not defined"));
@@ -613,7 +617,7 @@ avl_eval_rewrite (grad_request_t *req, grad_matching_rule_t *rule,
 }
 
 void
-rule_eval_rewrite (grad_request_t *req, grad_matching_rule_t *rule)
+rule_eval_rewrite (radiusd_request_t *req, grad_matching_rule_t *rule)
 {
 	avl_eval_rewrite (req, rule, rule->rhs);
 	avl_eval_rewrite (req, rule, rule->lhs);
@@ -626,7 +630,7 @@ rule_eval_rewrite (grad_request_t *req, grad_matching_rule_t *rule)
 /* Provide a support for backward-compatible attributes Replace-User-Name
    and Rewrite-Function */
 static void
-hints_eval_compat(grad_request_t *req, grad_avp_t *name_pair, grad_matching_rule_t *rule)
+hints_eval_compat(radiusd_request_t *req, grad_avp_t *name_pair, grad_matching_rule_t *rule)
 {
         grad_avp_t      *tmp;
 	
@@ -638,7 +642,7 @@ hints_eval_compat(grad_request_t *req, grad_avp_t *name_pair, grad_matching_rule
  
 		obstack_init(&hints_stk);
 		ptr = radius_xlate(&hints_stk, tmp->avp_strvalue,
-				   req, NULL);
+				   req->request, NULL);
 		if (ptr) 
 			grad_string_replace(&name_pair->avp_strvalue, ptr);
 		obstack_free(&hints_stk, NULL);
@@ -651,10 +655,10 @@ hints_eval_compat(grad_request_t *req, grad_avp_t *name_pair, grad_matching_rule
    based on the pattern of the username and the contents of the incoming
    request */
 int
-hints_setup(grad_request_t *req)
+hints_setup(radiusd_request_t *req)
 {
-        grad_avp_t      *request_pairs = req->request;
-        char            newname[AUTH_STRING_LEN];
+        grad_avp_t      *request_pairs = req->request->avlist;
+        char            newname[GRAD_STRING_LENGTH];
         grad_avp_t      *name_pair;
         grad_avp_t      *orig_name_pair;
         grad_avp_t      *tmp;
@@ -663,7 +667,7 @@ hints_setup(grad_request_t *req)
 	grad_iterator_t *itr;
 	
         /* Add Proxy-Replied pair if necessary */
-        switch (req->code) {
+        switch (req->request->code) {
         case RT_ACCESS_ACCEPT:
         case RT_ACCESS_REJECT:
         case RT_ACCOUNTING_RESPONSE:
@@ -777,7 +781,7 @@ hints_setup(grad_request_t *req)
                         grad_avp_free(name_pair);
         }
 
-        req->request = request_pairs;
+        req->request->avlist = request_pairs;
         
         return 0;
 }
@@ -790,7 +794,7 @@ hints_setup(grad_request_t *req)
  * See if the huntgroup matches.
  */
 int
-huntgroup_match(grad_request_t *req, char *huntgroup)
+huntgroup_match(radiusd_request_t *req, char *huntgroup)
 {
         grad_matching_rule_t *rule;
 	grad_iterator_t *itr;
@@ -820,7 +824,7 @@ huntgroup_match(grad_request_t *req, char *huntgroup)
  *         -1 on error.
  */
 int
-huntgroup_access(grad_request_t *radreq, grad_locus_t *loc)
+huntgroup_access(radiusd_request_t *radreq, grad_locus_t *loc)
 {
         grad_avp_t      *pair;
         grad_matching_rule_t   *rule;
@@ -929,8 +933,8 @@ client_lookup_ip(grad_uint32_t ipaddr)
                 return NULL;
         for (cl = grad_iterator_first(itr); cl; cl = grad_iterator_next(itr))
 		if (grad_ip_in_net_p(&cl->netdef, ipaddr)) {
-			char ipbuf[DOTTED_QUAD_LEN];
-			char maskbuf[DOTTED_QUAD_LEN];
+			char ipbuf[GRAD_IPV4_STRING_LENGTH];
+			char maskbuf[GRAD_IPV4_STRING_LENGTH];
 
 			debug(10,
 			      ("Found secret for %s/%s (%s): %s",
@@ -1139,7 +1143,7 @@ fallthrough(grad_avp_t *vp)
 int
 portcmp(grad_avp_t *check, grad_avp_t *request)
 {
-        char buf[AUTH_STRING_LEN];
+        char buf[GRAD_STRING_LENGTH];
         char *s, *p, *save;
         int lo, hi;
         int port = request->avp_lvalue;
@@ -1176,7 +1180,7 @@ uidcmp(grad_avp_t *check, char *username)
 }
 
 static int
-system_groupcmp(grad_request_t *req, char *groupname, char *username)
+system_groupcmp(radiusd_request_t *req, char *groupname, char *username)
 {
         struct passwd pw, *pwd;
         struct group *grp;
@@ -1202,7 +1206,7 @@ system_groupcmp(grad_request_t *req, char *groupname, char *username)
 
 /* Return 0 if user `username' is a member of group `groupname' */
 int
-groupcmp(grad_request_t *req, char *groupname, char *username)
+groupcmp(radiusd_request_t *req, char *groupname, char *username)
 {
         int retval;
 
@@ -1280,7 +1284,7 @@ server_attr(int attr)
 {
         int i;
 
-        for (i = 0; i < NITEMS(server_check_items); i++) 
+        for (i = 0; i < GRAD_NITEMS(server_check_items); i++) 
                 if (server_check_items[i] == attr)
                         return 1;       
         return 0;
@@ -1291,11 +1295,11 @@ server_attr(int attr)
  * Return 0 on match.
  */
 int
-paircmp(grad_request_t *request, grad_avp_t *check, char *pusername)
+paircmp(radiusd_request_t *request, grad_avp_t *check, char *pusername)
 {
         grad_avp_t *check_item = check;
         grad_avp_t *auth_item;
-        char username[AUTH_STRING_LEN];
+        char username[GRAD_STRING_LENGTH];
         int result = 0;
         int compare;
         char *save;
@@ -1319,7 +1323,7 @@ paircmp(grad_request_t *request, grad_avp_t *check, char *pusername)
                 /*
                  *      See if this item is present in the request.
                  */
-                for (auth_item = request->request; auth_item; 
+                for (auth_item = request->request->avlist; auth_item; 
                                 auth_item = auth_item->next) {
                         debug(30, ("trying %d", auth_item->attribute));
 
@@ -1369,7 +1373,7 @@ paircmp(grad_request_t *request, grad_avp_t *check, char *pusername)
                 
                 compare = 0;    /* default result */
                 switch (check_item->type) {
-                case TYPE_STRING:
+                case GRAD_TYPE_STRING:
                         switch (check_item->attribute) {
                         case DA_PREFIX:
                         case DA_SUFFIX:
@@ -1402,7 +1406,7 @@ paircmp(grad_request_t *request, grad_avp_t *check, char *pusername)
                         }
                         break;
 
-                case TYPE_INTEGER:
+                case GRAD_TYPE_INTEGER:
                         switch (check_item->attribute) {
                         case DA_USER_UID:
                                 compare = uidcmp(check_item, username);
@@ -1410,7 +1414,7 @@ paircmp(grad_request_t *request, grad_avp_t *check, char *pusername)
                         }
                         /*FALLTHRU*/
 			
-                case TYPE_IPADDR:
+                case GRAD_TYPE_IPADDR:
                         compare = auth_item->avp_lvalue - check_item->avp_lvalue;
                         break;
                         
@@ -1574,9 +1578,9 @@ wild_match(char *expr, char *name, char *return_name)
  * Match a username with a wildcard expression.
  */
 int
-matches(grad_request_t *req, char *name, grad_matching_rule_t *pl, char *matchpart)
+matches(radiusd_request_t *req, char *name, grad_matching_rule_t *pl, char *matchpart)
 {
-	memcpy(matchpart, name, AUTH_STRING_LEN);
+	memcpy(matchpart, name, GRAD_STRING_LENGTH);
         if (strncmp(pl->name, "DEFAULT", 7) == 0
 	    || wild_match(pl->name, name, matchpart) == 0)
                 return paircmp(req, pl->lhs, matchpart);
@@ -1683,14 +1687,14 @@ reload_data(enum reload_what what, int *do_radck)
         case reload_huntgroups:
                 grad_list_destroy(&huntgroups, matching_rule_free, NULL);
                 path = grad_mkfilename(radius_dir, RADIUS_HUNTGROUPS);
-                huntgroups = file_read(CF_HUNTGROUPS, path);
+                huntgroups = file_read(GRAD_CF_HUNTGROUPS, path);
                 grad_free(path);
                 break;
                 
         case reload_hints:
                 grad_list_destroy(&hints, matching_rule_free, NULL);
                 path = grad_mkfilename(radius_dir, RADIUS_HINTS);
-                hints = file_read(CF_HINTS, path);
+                hints = file_read(GRAD_CF_HINTS, path);
                 grad_free(path);
                 if (!use_dbm) 
                         *do_radck = 1;
@@ -1792,8 +1796,11 @@ dump_matching_rules(FILE *fp, char *header, grad_list_t *list)
 }
 
 int
-dump_user(FILE *fp, User_symbol *sym)
+dump_user(void *data, grad_symbol_t *symbol)
 {
+	FILE *fp = data;
+	User_symbol *sym = (User_symbol *) symbol;
+
         fprintf(fp, "\t%s:\n", sym->name);
         fprintf(fp, "\tlhs {\n");
 	grad_avl_fprint(fp, "\t\t", 1, sym->check);
@@ -1842,7 +1849,7 @@ void
 strip_username(int do_strip, char *name, grad_avp_t *check_item,
 	       char *stripped_name)
 {
-        char tmpname[AUTH_STRING_LEN];
+        char tmpname[GRAD_STRING_LENGTH];
         char *source_ptr = name;
         grad_avp_t *presuf_item, *tmp;
         
