@@ -87,8 +87,12 @@ static char *
 run_log_hook(const grad_request_t *req, const char *hook_name)
 {
 	char *result;
-	char nasbuf[MAX_LONGNAME];
-	
+	char nasbuf[GRAD_MAX_LONGNAME];
+
+	/* FIXME: Should make sure that the hook does not modify the
+	   request, either by passing rewrite_invoke a copy of the
+	   latter (expensive), or by providing some internal rewrite
+	   mechanism */
 	if (rewrite_invoke(String,
 			   &result,
 			   hook_name,
@@ -573,7 +577,7 @@ static struct category_def {
         int level;
 } cat_def;
 
-static struct keyword syslog_facility[] = {
+static grad_keyword_t syslog_facility[] = {
 	{ "user", 	LOG_USER },
 	{ "daemon", 	LOG_DAEMON },
 	{ "auth", 	LOG_AUTH },
@@ -588,7 +592,7 @@ static struct keyword syslog_facility[] = {
 	{ 0 }
 };
 
-static struct keyword syslog_priority[] = {
+static grad_keyword_t syslog_priority[] = {
 	{ "emerg", 	LOG_EMERG },
 	{ "alert", 	LOG_ALERT },
 	{ "crit", 	LOG_CRIT },
@@ -600,7 +604,7 @@ static struct keyword syslog_priority[] = {
 	{ 0 }
 };
 
-static struct keyword log_categories[] = {
+static grad_keyword_t log_categories[] = {
 	{ "main",       L_MAIN },
 	{ "auth",       L_AUTH },
 	{ "acct",       L_ACCT },
@@ -609,7 +613,7 @@ static struct keyword log_categories[] = {
 	{ 0 }
 };
 
-static struct keyword log_priorities[] = {
+static grad_keyword_t log_priorities[] = {
 	{ "emerg",      L_EMERG },
 	{ "alert",      L_ALERT },
 	{ "crit",       L_CRIT },
@@ -898,7 +902,7 @@ category_set_level(int argc, cfg_value_t *argv,
 {
 	int i;
 
-	clear_debug();
+	grad_clear_debug();
 	for (i = 1; i < argc; ) {
 		char *modname;
 		int level;
@@ -919,7 +923,7 @@ category_set_level(int argc, cfg_value_t *argv,
 				return 1;
 			level = argv[i++].v.number;
 		}
-		if (set_module_debug_level(modname, level)) {
+		if (grad_set_module_debug_level(modname, level)) {
 			grad_log(L_WARN,
 			         _("%s:%d: no such module name: %s"),
 			         cfg_filename, cfg_line_num, modname);
