@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2003 Free Software Foundation, Inc.
+   Copyright (C) 2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
@@ -19,8 +19,6 @@
 
 /* RPP is a Radius Process Pool */
 
-#define RADIUS_MODULE_RPP_C
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -28,17 +26,52 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <errno.h>
+
 #include <radiusd.h>
-#include <list.h>
+
+static int rpp_stdin;
+static int rpp_stdout;
 
 /* Process intercommunication primitives */
 
+enum process_status {
+	process_busy,      /* Process is busy handling a request */
+	process_ready,     /* Precess is idle and ready for input */
+	process_finished   /* Process has finished */
+};
+
 typedef struct {
-	pid_t pid;   
-	int p[2];
-	int ready;
+	pid_t pid;    /* PID of the handler process */
+	int p[2];     /* IPC descriptors. p[0] - input, p[1] - output */
+	enum process_status status; /* Current process status */
+	int exit_status;  /* Process exit status if status==process_finished */
 } rpp_proc_t;
 
+
+/* Low-level calls */
+
+/* Recompute the timeout TVAL taking into account the time elapsed since
+   START */
+static int
+recompute_timeout(struct timeval *start, struct timeval *tval)
+{
+	struct timeval now, diff;
+	
+	gettimeofday(&now, NULL);
+	timersub(&now, start, &diff);
+	if (timercmp(&diff, tval, <)) {
+		struct timeval tmp;
+		timersub(tval, &diff, &tmp);
+		*tval = tmp;
+		return 0;
+	}
+	return 1;
+}
+
+/* Write into the pipe (represented by FD) SIZE bytes from PTR. TV
+   sets timeout. TV==NULL means no timeout is imposed.
+
+   Returns number of bytes written */
 static int
 pipe_write(int fd, void *ptr, size_t size, struct timeval *tv)
 {
@@ -47,19 +80,27 @@ pipe_write(int fd, void *ptr, size_t size, struct timeval *tv)
 	else {
 		char *data = ptr;
 		int rc;
-		struct timeval to;
+		struct timeval tval, start;
 		fd_set wr_set;
 		size_t n;
-		
+
+		tval = *tv;
+		gettimeofday(&start, NULL);
 		for (n = 0; n < size;) {
-			to = *tv;
+			struct timeval to;
+			
 			FD_ZERO(&wr_set);
 			FD_SET(fd, &wr_set);
+
+			if (recompute_timeout (&start, &tval))
+				break;
+			to = tval;
+			
 			rc = select(fd + 1, NULL, &wr_set, NULL, &to);
 			if (rc == 0)
 				break;
 			else if (rc < 0) {
-				if (errno == EINTR)
+				if (errno == EINTR) 
 					continue;
 				break;
 			} else if (rc > 0) {
@@ -74,6 +115,10 @@ pipe_write(int fd, void *ptr, size_t size, struct timeval *tv)
 	}
 }
 
+/* Read from the pipe (represented by FD) at most SIZE bytes into PTR. TV
+   sets timeout. TV==NULL means no timeout is imposed.
+
+   Returns number of bytes read */
 static int
 pipe_read(int fd, void *ptr, size_t size, struct timeval *tv)
 {
@@ -93,14 +138,22 @@ pipe_read(int fd, void *ptr, size_t size, struct timeval *tv)
 		} while (size > 0);
 		return rdbytes;
 	} else {
-		struct timeval to;
+		struct timeval tval, start;
 		fd_set rd_set;
 		size_t n;
 		
+		tval = *tv;
+		gettimeofday(&start, NULL);
 		for (n = 0; n < size;) {
-			to = *tv;
+			struct timeval to;
+
 			FD_ZERO(&rd_set);
 			FD_SET(fd, &rd_set);
+
+			if (recompute_timeout (&start, &tval))
+				break;
+			to = tval;
+
 			rc = select(fd + 1, &rd_set, NULL, NULL, &to);
 			if (rc == 0)
 				break;
@@ -120,14 +173,20 @@ pipe_read(int fd, void *ptr, size_t size, struct timeval *tv)
 	}
 }
 
+
+/* RPP layer I/O. Each packet transmitted over a pipe is preceeded by
+   its length (a size_t number in host order) */
+
+/* Read SIZE bytes from the pipe (FD) into DATA. TV sets timeout */
 static int
 rpp_fd_read(int fd, void *data, size_t size, struct timeval *tv)
 {
-	size_t sz, nbytes;
+	size_t sz, nbytes = 0;
 
 	sz = pipe_read(fd, &nbytes, sizeof(nbytes), tv);
 	if (sz == 0)
 		return 0; /* eof */
+	debug(100,("nbytes=%lu",nbytes));
 	if (sz != sizeof(nbytes)) 
 		return -1;
 	sz = nbytes > size ? size : nbytes;
@@ -135,15 +194,20 @@ rpp_fd_read(int fd, void *data, size_t size, struct timeval *tv)
 		return -2;
 	for (;nbytes > size; nbytes--) {
 		char c;
-		pipe_read(fd, &c, 1, tv);
+		if (pipe_read(fd, &c, 1, tv) != 1) {
+			debug(1,("pipe_read failed"));
+			return -3;
+		}
 	}
 	
 	return sz;
 }
 
+/* Write SIZE bytes from DATA to the pipe FD. TV sets timeout */
 static int
 rpp_fd_write(int fd, void *data, size_t size, struct timeval *tv)
 {
+	debug(100,("size=%lu",size));
 	if (pipe_write(fd, &size, sizeof(size), tv) != sizeof(size))
 		return -1;
 	if (pipe_write(fd, data, size, tv) != size)
@@ -151,6 +215,12 @@ rpp_fd_write(int fd, void *data, size_t size, struct timeval *tv)
 	return size;
 }
 
+
+
+/* Start a handler process. PROC_MAIN is the handler function (process'
+   main loop. DATA is passed to PROC_MAIN verbatim.
+
+   On success return 0 and fill in PROC structure. */
 int
 rpp_start_process(rpp_proc_t *proc, int (*proc_main)(void *), void *data)
 {
@@ -159,34 +229,32 @@ rpp_start_process(rpp_proc_t *proc, int (*proc_main)(void *), void *data)
 	pid_t pid;
 	
 	if (pipe(inp)) {
-		radlog(L_ERR, "pipe(inp): %s", strerror(errno));
+		grad_log(L_ERR, "pipe(inp): %s", strerror(errno));
 		return -1;
 	}
 	
 	if (pipe(outp)) {
-		radlog (L_ERR, "pipe(outp): %s", strerror(errno));
+		grad_log (L_ERR, "pipe(outp): %s", strerror(errno));
 		return -1;
 	}
 
 	pid = fork();
 	if (pid == -1) {
-		radlog (L_ERR, "fork: %s", strerror(errno));
+		grad_log (L_ERR, "fork: %s", strerror(errno));
 		return -1;
 	}
 	if (pid == 0) {
 		/* Child */
-
 		/* Close remote side of pipes */
 		close(inp[0]);
 		close(outp[1]);
 		/* Close stdio */
 		close(0);
 		close(1);
-		
-		/* Duplicate i/o channels to stdio */
-		dup(outp[0]);  /* This becomes standard in */
-		dup(inp[1]);   /* and this standard out */
 
+		rpp_stdin = outp[0];
+		rpp_stdout = inp[1];
+		
 		/* Run the main process */
 		exit(proc_main(data));
 	}
@@ -198,90 +266,139 @@ rpp_start_process(rpp_proc_t *proc, int (*proc_main)(void *), void *data)
 	proc->pid = pid;
 	proc->p[0] = inp[0];
 	proc->p[1] = outp[1];
-	proc->ready = 1;
+	proc->status = process_ready;
 	return 0;
 }
 
   
 
-static RAD_LIST *process_list; /* List of rpp_proc_t */
+static grad_list_t *process_list; /* List of rpp_proc_t */
 
+/* Look up the rpp_proc_t whose input descriptor is FD */
 rpp_proc_t *
 rpp_lookup_fd(int fd)
 {
 	rpp_proc_t *p;
-	ITERATOR *itr = iterator_create(process_list);
-	for (p = iterator_first(itr); p; p = iterator_next(itr))
+	grad_iterator_t *itr = grad_iterator_create(process_list);
+	for (p = grad_iterator_first(itr); p; p = grad_iterator_next(itr))
 		if (p->p[0] == fd)
 			break;
-	iterator_destroy(&itr);
+	grad_iterator_destroy(&itr);
 	return p;
 }
 
+/* Find an rpp_proc_t ready for handling the reqest. If none is found,
+   start a new one. PROC_MAIN and DATA have the same meaning as in
+   rpp_start_process() */
 rpp_proc_t *
 rpp_lookup_ready(int (*proc_main)(void *), void *data)
 {
 	rpp_proc_t *p;
 
 	if (process_list) {
-		ITERATOR *itr = iterator_create(process_list);
-		for (p = iterator_first(itr); p && !p->ready;
-		     p = iterator_next(itr))
+		grad_iterator_t *itr = grad_iterator_create(process_list);
+		for (p = grad_iterator_first(itr);
+		     p && p->status != process_ready;
+		     p = grad_iterator_next(itr))
 			;
-		iterator_destroy(&itr);
+		grad_iterator_destroy(&itr);
 	} else {
-		process_list = list_create();
+		process_list = grad_list_create();
 		p = NULL;
 	}
 	
 	if (!p) {
 		rpp_proc_t proc;
-		if (list_count(process_list) == max_children) 
+		if (grad_list_count(process_list) == max_children) 
 			return NULL;
 		if (rpp_start_process(&proc, proc_main, data)) 
 			return NULL;
 		radiusd_register_input_fd("rpp", proc.p[0], NULL);
-		p = emalloc(sizeof(*p));
+		p = grad_emalloc(sizeof(*p));
 		*p = proc;
-		list_append(process_list, p);
+		grad_list_append(process_list, p);
 	}
 	return p;
 }
 
+/* Find the rpp_proc_t with a given PID */
 rpp_proc_t *
 rpp_lookup_pid(pid_t pid)
 {
 	rpp_proc_t *p;
-	ITERATOR *itr = iterator_create(process_list);
-	for (p = iterator_first(itr); p; p = iterator_next(itr)) {
+	grad_iterator_t *itr = grad_iterator_create(process_list);
+	for (p = grad_iterator_first(itr); p; p = grad_iterator_next(itr)) {
 		if (p->pid == pid)
 			break;
 	}
-	iterator_destroy(&itr);
+	grad_iterator_destroy(&itr);
 	return p;
 }
-	
+
+/* Remove the given rpp_proc_t from the list. The underlying process
+   is supposed to have finished. No diagnostics is output. */
 static void
 _rpp_remove(rpp_proc_t *p)
 {
 	close(p->p[0]);
 	close(p->p[1]);
 	radiusd_close_channel(p->p[0]);
-	if (list_remove(process_list, p, NULL))
-		efree(p);
+	if (grad_list_remove(process_list, p, NULL))
+		grad_free(p);
 }
 
+/* Remove the given rpp_proc_t from the list. Output diagnostics about
+   exit status of the handler */
 void
-rpp_remove(pid_t pid)
+rpp_remove(rpp_proc_t *p)
+{
+	char buffer[128];
+	format_exit_status(buffer, sizeof buffer, p->exit_status);
+	grad_log(L_NOTICE, _("child %lu %s"),
+		 (unsigned long) p->pid, buffer);
+	_rpp_remove(p);
+}
+
+/* Remove the rpp entry with given PID */
+void
+rpp_remove_pid(pid_t pid)
 {
 	rpp_proc_t *p = rpp_lookup_pid(pid);
 	if (p)
-		_rpp_remove(p);
+		rpp_remove(p);
+}
+
+/* Reflect the change of state of the rpp handler process.
+
+   The function is safe to be used from signal handlers */
+void
+rpp_status_changed(pid_t pid, int exit_status)
+{
+	rpp_proc_t *p = rpp_lookup_pid(pid);
+	if (p) {
+		p->status = process_finished;
+		p->exit_status = exit_status;
+	}
+}
+
+/* Collect rpp_proc's whose handler have exited. Free any associated
+   resources */
+void
+rpp_collect_exited()
+{
+	rpp_proc_t *p;
+	grad_iterator_t *itr = grad_iterator_create(process_list);
+	for (p = grad_iterator_first(itr); p; p = grad_iterator_next(itr)) {
+		if (p->status == process_finished)
+			rpp_remove(p);
+	}
+	grad_iterator_destroy(&itr);
 }
 
 
 static int rpp_request_handler(void *arg);
 
+/* Return 1 if rpp with the given PID is ready for input */
 int
 rpp_ready(pid_t pid)
 {
@@ -290,38 +407,49 @@ rpp_ready(pid_t pid)
 			return 1;
 	} else {
 		rpp_proc_t *p;
-		ITERATOR *itr = iterator_create(process_list);
+		grad_iterator_t *itr = grad_iterator_create(process_list);
 
-		for (p = iterator_first(itr); p; p = iterator_next(itr)) {
+		for (p = grad_iterator_first(itr); p; p = grad_iterator_next(itr)) {
 			if (p->pid == pid) {
 				break;
 			}
 		}
-	        iterator_destroy(&itr);
-		if (p && p->ready)
+	        grad_iterator_destroy(&itr);
+		if (p && p->status == process_ready)
 			return 1;
 	}
 	return 0;
 }
 
+/* Traverse the rpp list until FUN returns non-zero or there are no
+   more busy handlers. While traversing, remove finished handlers. */
 void
 rpp_flush(int (*fun)(void*), void *closure)
 {
 	time_t t;
 	unsigned count;
-	ITERATOR *itr = iterator_create(process_list);
+	grad_iterator_t *itr = grad_iterator_create(process_list);
 
 	time(&t);
 
 	do {
 		rpp_proc_t *p;
-		for (count = 0, p = iterator_first(itr);
+		for (count = 0, p = grad_iterator_first(itr);
 		     p;
-		     p = iterator_next(itr))
-			if (!p->ready)
+		     p = grad_iterator_next(itr))
+			switch (p->status) {
+			case process_ready:
+				break;
+
+			case process_busy:
 				count++;
+				break;
+
+			case process_finished:
+				rpp_remove (p);
+			}
 	} while (count > 0 && (*fun)(closure) == 0);
-	iterator_destroy(&itr);
+	grad_iterator_destroy(&itr);
 }
 
 static int
@@ -331,7 +459,8 @@ _kill_itr(void *item, void *data)
 	kill(p->pid, *(int*)data);
 	return 0;
 }
-	
+
+/* Deliver signal SIGNO to the handler with pid PID */
 int
 rpp_kill(pid_t pid, int signo)
 {
@@ -343,24 +472,28 @@ rpp_kill(pid_t pid, int signo)
 		} else
      		        return 1;
 	} else 
-		list_iterate(process_list, _kill_itr, &signo);
+		grad_list_iterate(process_list, _kill_itr, &signo);
 	return 0;
 }
 
+/* Slay the child */
 static void
 _rpp_slay(rpp_proc_t *p)
 {
-	radlog(L_NOTICE, _("Killing unresponding process %lu"), (u_long) p->pid);
+	grad_log(L_NOTICE, _("Killing unresponding process %lu"), (u_long) p->pid);
 	kill(p->pid, SIGKILL);
 	_rpp_remove(p);
 }
 
+/* Return number of the handlers registered in the process list */
 size_t
 rpp_count()
 {
-	return list_count(process_list);
+	return grad_list_count(process_list);
 }
 
+
+
 struct rpp_request {
 	int type;                  /* Request type */
 	struct sockaddr_in addr;   /* Sender address */
@@ -400,7 +533,7 @@ rpp_forward_request(REQUEST *req)
 	frq.fd = req->fd;
 	frq.size = req->rawsize;
 	
-	p->ready = 0;
+	p->status = process_busy;
 	req->child_id = p->pid;
 
 	if (radiusd_write_timeout) {
@@ -443,7 +576,7 @@ sig_handler(int sig)
 		break;
 		
 	case SIGALRM:
-		radlog(L_INFO, _("Child exiting on timeout."));
+		grad_log(L_INFO, _("Child exiting on timeout."));
 		/*FALLTHRU*/
 		
 	case SIGTERM:
@@ -461,7 +594,7 @@ sig_handler(int sig)
 	default:
 		abort();
 	}
-	rad_reset_signal(sig, sig_handler);
+	grad_reset_signal(sig, sig_handler);
 }
 
 /* Main loop for a child process */
@@ -475,7 +608,7 @@ rpp_request_handler(void *arg ARG_UNUSED)
 	REQUEST *req;
 
 	radiusd_signal_init(sig_handler);
-	rad_set_signal(SIGALRM, sig_handler);
+	grad_set_signal(SIGALRM, sig_handler);
 	request_init_queue();
 #ifdef USE_SERVER_GUILE
         scheme_redirect_output();
@@ -486,30 +619,31 @@ rpp_request_handler(void *arg ARG_UNUSED)
 		int len;
 
 		alarm(process_timeout);
-		len = rpp_fd_read(0, &frq, sizeof frq, NULL);
+		len = rpp_fd_read(rpp_stdin, &frq, sizeof frq, NULL);
 		alarm(0);
 		if (len != sizeof frq) {
 			if (errno == EINTR)
 				continue;
-			radlog(L_ERR,
-			       _("Child received malformed header (len = %d, error = %s)"),
-			       len, strerror(errno));
+			grad_log(L_ERR,
+			         _("Child received malformed header (len = %d, error = %s)"),
+			         len, strerror(errno));
 			radiusd_exit0();
 		}
 
 		if (datasize < frq.size) {
 			datasize = frq.size;
-			data = erealloc(data, datasize);
+			data = grad_erealloc(data, datasize);
 		}
 		
-		if (rpp_fd_read(0, data, frq.size, NULL) != frq.size) {
-			radlog(L_ERR,
-			       _("Child received malformed data"));
+		if (rpp_fd_read(rpp_stdin, data, frq.size, NULL) != frq.size) {
+			grad_log(L_ERR,
+			         _("Child received malformed data"));
 			radiusd_exit0();
 		}
-		
+
 		req = request_create(frq.type, frq.fd, &frq.addr,
 				     data, frq.size);
+
 		req->status = RS_COMPLETED;
 		rc = request_handle(req, request_respond);
 			
@@ -517,7 +651,7 @@ rpp_request_handler(void *arg ARG_UNUSED)
 		debug(1, ("notifying the master"));
 		repl.code = RPP_COMPLETE;
 		repl.size = 0;
-		rpp_fd_write(1, &repl, sizeof repl, NULL);
+		rpp_fd_write(rpp_stdout, &repl, sizeof repl, NULL);
 		if (rc)
 			request_free(req);
 	}
@@ -532,8 +666,8 @@ rpp_input_handler(int fd, void *data)
 	rpp_proc_t *p = rpp_lookup_fd(fd);
 	struct timeval tv, *tvp;
 	int sz;
-	
-	insist(p != NULL);
+
+	grad_insist(p != NULL);
 	
 	if (radiusd_read_timeout) {
 		tv.tv_sec = radiusd_read_timeout;
@@ -547,21 +681,21 @@ rpp_input_handler(int fd, void *data)
 		void *data = NULL;
 
 		if (repl.size) {
-			data = emalloc(repl.size);
+			data = grad_emalloc(repl.size);
 			if (rpp_fd_read(fd, data, repl.size, tvp)
 			    != repl.size) {
 				_rpp_slay(p);
-				efree(data);
+				grad_free(data);
 				return 1;
 			}
 		}
 		
 		if (p) {
 		        debug(1, ("updating pid %d", p->pid));
-			p->ready = 1;
+			p->status = process_ready;
 			request_update(p->pid, RS_COMPLETED, data);
 		} 
-		efree(data);
+		grad_free(data);
 	} else if (sz != 0) {
 		_rpp_slay(p);
 		return 1;
@@ -578,8 +712,8 @@ rpp_update(void *data, size_t size)
 
 	repl.code = RPP_UPDATE;
 	repl.size = size;
-	rpp_fd_write(1, &repl, sizeof repl, NULL);
-	rpp_fd_write(1, data, size, NULL);
+	rpp_fd_write(rpp_stdout, &repl, sizeof repl, NULL);
+	rpp_fd_write(rpp_stdout, data, size, NULL);
 	return 0;
 }
 
@@ -592,7 +726,9 @@ rpp_input_close(int fd, void *data)
 	return 0;
 }
 
+
 
+/* Don't use it. It's a debugging hook */
 int
 wd()
 {
