@@ -83,7 +83,7 @@ static char *cur_line, *cur_ptr;
 static struct obstack parse_stack;
 static int stmt_type;
 
-struct keyword sql_keyword[] = {
+static grad_keyword_t sql_keyword[] = {
         { "server",             STMT_SERVER },
         { "port",               STMT_PORT },
         { "login",              STMT_LOGIN },
@@ -119,7 +119,7 @@ struct keyword sql_keyword[] = {
 static char *
 sql_keyword_name(int kw)
 {
-	struct keyword *p;
+	grad_keyword_t *p;
 
 	for (p = sql_keyword; p->name; p++)
 		if (p->tok == kw)
@@ -454,7 +454,6 @@ radiusd_sql_config()
         }
 
         obstack_free(&parse_stack, NULL);
-
         fclose(sqlfd);
 
         sql_check_config(sqlfile, &new_cfg);
@@ -513,6 +512,12 @@ sql_check_config(const char *filename, SQL_cfg *cfg)
 
 	/* Check general SQL setup */
 	if (cfg->active[SQL_AUTH] || cfg->active[SQL_ACCT]) {
+		if (disp_sql_interface_index(NULL) == 0) {
+			grad_log(L_ERR, _("%s: SQL interface not specified"),
+				 filename);
+			doacct = doauth = 0;
+		}
+
 		if (!cfg->server) {
 			missing_statement(L_ERR, filename, "server");
 			doacct = doauth = 0;
@@ -728,7 +733,8 @@ attach_sql_connection(int type)
                 debug(1, ("allocating new %d sql connection", type));
 
                 conn = grad_emalloc(sizeof(struct sql_connection));
-                conn->connected = 0;
+		conn->cfg = &sql_cfg;
+		conn->connected = 0;
                 conn->last_used = now;
                 conn->type = type;
 
@@ -790,7 +796,7 @@ radiusd_sql_cleanup(int type, void *req ARG_UNUSED)
  * Perform normal accounting
  */ 
 void
-radiusd_sql_acct(grad_request_t *radreq)
+radiusd_sql_acct(radiusd_request_t *radreq)
 {
         int rc, count;
         int status;
@@ -804,9 +810,10 @@ radiusd_sql_acct(grad_request_t *radreq)
         if (!sql_cfg.active[SQL_ACCT])
                 return;
 
-        if (!(pair = grad_avl_find(radreq->request, DA_ACCT_STATUS_TYPE))) {
+        if (!(pair = grad_avl_find(radreq->request->avlist,
+				   DA_ACCT_STATUS_TYPE))) {
                 /* should never happen!! */
-                grad_log_req(L_ERR, radreq,
+                grad_log_req(L_ERR, radreq->request,
                              _("no Acct-Status-Type attribute in rad_sql_acct()"));
                 return;
         }
@@ -822,7 +829,7 @@ radiusd_sql_acct(grad_request_t *radreq)
                         break;
                 query = radius_xlate(&stack,
                                      sql_cfg.query[acct_start_query],
-                                     radreq, NULL);
+                                     radreq->request, NULL);
                 rc = disp_sql_query(conn, query, NULL);
                 sqllog(rc, query);
 		log_facility = 0;
@@ -834,7 +841,7 @@ radiusd_sql_acct(grad_request_t *radreq)
                         break;
                 query = radius_xlate(&stack,
                                      sql_cfg.query[acct_stop_query],
-                                     radreq, NULL);
+                                     radreq->request, NULL);
                 rc = disp_sql_query(conn, query, &count);
                 sqllog(rc, query);
                 if (rc == 0 && count != 1) 
@@ -847,7 +854,7 @@ radiusd_sql_acct(grad_request_t *radreq)
                         break;
                 query = radius_xlate(&stack,
                                      sql_cfg.query[acct_nasup_query],
-                                     radreq, NULL);
+                                     radreq->request, NULL);
                 rc = disp_sql_query(conn, query, &count);
                 sqllog(rc, query);
 		if (rc == 0)
@@ -860,7 +867,7 @@ radiusd_sql_acct(grad_request_t *radreq)
                         break;
                 query = radius_xlate(&stack,
                                      sql_cfg.query[acct_nasdown_query],
-                                     radreq, NULL);
+                                     radreq->request, NULL);
                 rc = disp_sql_query(conn, query, &count);
                 sqllog(rc, query);
 		if (rc == 0)
@@ -873,7 +880,7 @@ radiusd_sql_acct(grad_request_t *radreq)
                         break;
                 query = radius_xlate(&stack,
                                      sql_cfg.query[acct_keepalive_query],
-                                     radreq, NULL);
+                                     radreq->request, NULL);
                 rc = disp_sql_query(conn, query, &count);
                 sqllog(rc, query);
                 if (rc == 0 && count != 1) 
@@ -883,7 +890,7 @@ radiusd_sql_acct(grad_request_t *radreq)
         }
 
 	if (log_facility) {
-		grad_log_req(log_facility, radreq,
+		grad_log_req(log_facility, radreq->request,
 			     ngettext("%s updated %d record",
 				      "%s updated %d records",
 				      count),
@@ -896,7 +903,7 @@ radiusd_sql_acct(grad_request_t *radreq)
 
 
 char *
-radiusd_sql_pass(grad_request_t *req, char *authdata)
+radiusd_sql_pass(radiusd_request_t *req, char *authdata)
 {
         char *mysql_passwd;
         struct sql_connection *conn;
@@ -910,14 +917,15 @@ radiusd_sql_pass(grad_request_t *req, char *authdata)
         }
         
         if (authdata) {
-                grad_avl_add_pair(&req->request,
+                grad_avl_add_pair(&req->request->avlist,
 				  grad_avp_create_string(DA_AUTH_DATA,
 							 authdata));
         }
         
         obstack_init(&stack);
-        query = radius_xlate(&stack, sql_cfg.query[auth_query], req, NULL);
-        grad_avl_delete(&req->request, DA_AUTH_DATA);
+        query = radius_xlate(&stack, sql_cfg.query[auth_query],
+			     req->request, NULL);
+        grad_avl_delete(&req->request->avlist, DA_AUTH_DATA);
         
         conn = attach_sql_connection(SQL_AUTH);
         mysql_passwd = disp_sql_getpwd(conn, query);
@@ -931,7 +939,7 @@ radiusd_sql_pass(grad_request_t *req, char *authdata)
 }
 
 int
-radiusd_sql_checkgroup(grad_request_t *req, char *groupname)
+radiusd_sql_checkgroup(radiusd_request_t *req, char *groupname)
 {
         int   rc = -1;
         struct sql_connection *conn;
@@ -949,7 +957,8 @@ radiusd_sql_checkgroup(grad_request_t *req, char *groupname)
 
         obstack_init(&stack);
 	
-        query = radius_xlate(&stack, sql_cfg.query[group_query], req, NULL);
+        query = radius_xlate(&stack, sql_cfg.query[group_query],
+			     req->request, NULL);
 	res = sql_cache_lookup(conn, query);
 	if (!res) {
 		res = sql_cache_retrieve(conn, query);
@@ -1000,7 +1009,7 @@ rad_sql_retrieve_pairs(struct sql_connection *conn,
 		if (op_too) {
 			char *opstr = res->tuple[i][2];
                         op = grad_str_to_op(opstr);
-                        if (op == NUM_OPERATORS) {
+                        if (op == grad_operator_invalid) {
                                 grad_log(L_NOTICE,
                                          _("SQL: invalid operator: %s"), opstr);
                                 continue;
@@ -1025,7 +1034,7 @@ rad_sql_retrieve_pairs(struct sql_connection *conn,
 }
 			
 int
-radiusd_sql_reply_attr_query(grad_request_t *req, grad_avp_t **reply_pairs)
+radiusd_sql_reply_attr_query(radiusd_request_t *req, grad_avp_t **reply_pairs)
 {
         struct sql_connection *conn;
         char *query;
@@ -1039,7 +1048,7 @@ radiusd_sql_reply_attr_query(grad_request_t *req, grad_avp_t **reply_pairs)
         obstack_init(&stack);
 
         query = radius_xlate(&stack, sql_cfg.query[reply_attr_query],
-			     req, NULL);
+			     req->request, NULL);
         rc = rad_sql_retrieve_pairs(conn, query, reply_pairs, 0);
 
         obstack_free(&stack, NULL);
@@ -1047,7 +1056,7 @@ radiusd_sql_reply_attr_query(grad_request_t *req, grad_avp_t **reply_pairs)
 }
 
 int
-radiusd_sql_check_attr_query(grad_request_t *req, grad_avp_t **return_pairs)
+radiusd_sql_check_attr_query(radiusd_request_t *req, grad_avp_t **return_pairs)
 {
         struct sql_connection *conn;
         char *query;
@@ -1061,7 +1070,7 @@ radiusd_sql_check_attr_query(grad_request_t *req, grad_avp_t **return_pairs)
         obstack_init(&stack);
 	
         query = radius_xlate(&stack, sql_cfg.query[check_attr_query],
-			     req, NULL);
+			     req->request, NULL);
         rc = rad_sql_retrieve_pairs(conn, query, return_pairs, 1);
         
         obstack_free(&stack, NULL);
@@ -1082,7 +1091,7 @@ sql_auth_avail_p(const char **msg)
  * Auth failure trigger
  */
 void
-radiusd_sql_auth_result_query(grad_request_t *req, int fail)
+radiusd_sql_auth_result_query(radiusd_request_t *req, int fail)
 {
         struct sql_connection *conn;
         char *query;
@@ -1097,7 +1106,7 @@ radiusd_sql_auth_result_query(grad_request_t *req, int fail)
         obstack_init(&stack);
 	
         query = radius_xlate(&stack, sql_cfg.query[q],
-			     req, NULL);
+			     req->request, NULL);
 	rc = disp_sql_query(conn, query, NULL);
 	sqllog(rc, query);
         obstack_free(&stack, NULL);
@@ -1144,7 +1153,7 @@ rad_sql_retrieve_sessions(struct sql_connection *conn,
 
 int
 sql_mlc_collect(const char *query_template,
-		grad_request_t *request,
+		radiusd_request_t *request,
 		grad_list_t **sess_list)
 {
         struct sql_connection *conn;
@@ -1155,7 +1164,8 @@ sql_mlc_collect(const char *query_template,
         conn = attach_sql_connection(SQL_ACCT);
         obstack_init(&stack);
 	
-        query = radius_xlate(&stack, query_template, request, NULL);
+        query = radius_xlate(&stack, (char*) query_template,
+			     request->request, NULL);
         rc = rad_sql_retrieve_sessions(conn, query, sess_list);
 	obstack_free(&stack, NULL);
         return rc == 0;
@@ -1169,7 +1179,7 @@ rad_sql_mlc_enabled_p()
 }
 
 int
-rad_sql_mlc_collect_user(char *name, grad_request_t *request,
+rad_sql_mlc_collect_user(char *name, radiusd_request_t *request,
 			 grad_list_t **sess_list)
 {
         if (sql_cfg.active[SQL_ACCT] == 0 || !sql_cfg.query[mlc_user_query])
@@ -1179,7 +1189,7 @@ rad_sql_mlc_collect_user(char *name, grad_request_t *request,
 }
 
 int
-rad_sql_mlc_collect_realm(grad_request_t *request, grad_list_t **sess_list)
+rad_sql_mlc_collect_realm(radiusd_request_t *request, grad_list_t **sess_list)
 {
         if (sql_cfg.active[SQL_ACCT] == 0 || !sql_cfg.query[mlc_realm_query])
                 return 1;
@@ -1205,12 +1215,12 @@ rad_sql_mlc_close(struct radutmp *up)
 	obstack_init(&stack);
 	/* Create a temporary request */
 	req = grad_request_alloc();
-	grad_avl_add_pair(&req->request,
+	grad_avl_add_pair(&req->avlist,
 			  grad_avp_create_string(DA_USER_NAME, up->login));
-	grad_avl_add_pair(&req->request,
+	grad_avl_add_pair(&req->avlist,
 			  grad_avp_create_integer(DA_NAS_PORT_ID,
 						  up->nas_port));
-	grad_avl_add_pair(&req->request,
+	grad_avl_add_pair(&req->avlist,
 			  grad_avp_create_string(DA_ACCT_SESSION_ID,
 						 up->session_id));
 	query = radius_xlate(&stack, sql_cfg.query[query_index], req, NULL);
@@ -1237,6 +1247,7 @@ sql_init()
 			    rad_sql_mlc_collect_realm,
 			    rad_sql_mlc_close,
 			    rad_sql_mlc_enabled_p);
+	disp_init();
 }
 
 
