@@ -1,5 +1,5 @@
 /* This file is part of GNU Radius.
-   Copyright (C) 2000,2001,2002,2003 Free Software Foundation, Inc.
+   Copyright (C) 2000,2001,2002,2003,2004 Free Software Foundation, Inc.
 
    Written by Sergey Poznyakoff
 
@@ -17,7 +17,6 @@
    along with GNU Radius; if not, write to the Free Software Foundation, 
    Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
 
-#define RADIUS_MODULE_POSTGRES_C
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -27,10 +26,9 @@
 #include <sys/time.h>
 #include <sys/types.h>
 #include <unistd.h>
-#include <radius.h>
-#include <radsql.h>
 
-#ifdef USE_SQL_POSTGRES
+#include <common.h>
+#include <radsql.h>
 
 #include <libpq-fe.h>
 
@@ -49,27 +47,28 @@ rad_postgres_reconnect(int type, struct sql_connection *conn)
         
         switch (type) {
         case SQL_AUTH:
-                dbname = sql_cfg.auth_db;
+                dbname = conn->cfg->auth_db;
                 break;
         case SQL_ACCT:
-                dbname = sql_cfg.acct_db;
+                dbname = conn->cfg->acct_db;
                 break;
         }
 
-        if (sql_cfg.port == 0)
+        if (conn->cfg->port == 0)
                 portstr = NULL;
         else {
                 portstr = portbuf;
-                snprintf(portbuf, sizeof(portbuf), "%d", sql_cfg.port);
+                snprintf(portbuf, sizeof(portbuf), "%d", conn->cfg->port);
         }
                 
-        pgconn = PQsetdbLogin(sql_cfg.server, portstr, NULL, NULL,
+        pgconn = PQsetdbLogin(conn->cfg->server, portstr, NULL, NULL,
                               dbname,
-                              sql_cfg.login, sql_cfg.password);
+                              conn->cfg->login, conn->cfg->password);
         
         if (PQstatus(pgconn) == CONNECTION_BAD) {
-                radlog(L_ERR,
-                       _("PQconnectStart failed: %s"), PQerrorMessage(pgconn));
+                grad_log(L_ERR,
+                         _("PQconnectStart failed: %s"), 
+                         PQerrorMessage(pgconn));
                 PQfinish(pgconn);
                 return -1;
         }
@@ -104,26 +103,52 @@ rad_postgres_query(struct sql_connection *conn, char *query, int *return_count)
         
         res = PQexec((PGconn*)conn->data, query);
         if (res == NULL) {
-                radlog(L_ERR,
-                       "PQexec: %s",
-                       PQerrorMessage((PGconn*)conn->data));
+                grad_log(L_ERR,
+                         "PQexec: %s",
+                         PQerrorMessage((PGconn*)conn->data));
+		rad_postgres_disconnect(conn, 0);
                 return -1;
         }
         
         stat = PQresultStatus(res);
 
-        debug(1,
-              ("status: %s",
-              PQresStatus(stat)));
+        debug(1, ("status: %s", PQresStatus(stat)));
 
-        if (stat == PGRES_COMMAND_OK) {
+	switch (stat) {
+	case PGRES_COMMAND_OK:
                 if (return_count)
                         *return_count = atoi(PQcmdTuples(res));
                 rc = 0;
-        } else {
-                radlog(L_ERR,
-                       ("PQexec returned %s"),
-                       PQresStatus(stat));
+		break;
+
+	case PGRES_TUPLES_OK:
+		/* Oleg Gawriloff writes:
+
+		   "Sometimes Acct-Start does not reach radiusd, whereas
+		    subsequent Acct-Alive or Acct-Stop does. In this case
+		    radiusd runs acct_alive_query or acct_stop_query which
+		    try to update non-existing data. So, we chose to use
+		    a stored procedure in these queries, such that updates
+		    the record if it already exists or inserts a new record
+		    otherwise. The procedure, however, returns
+		    PGRES_TUPLES_OK, instead of PGRES_COMMAND_OK which
+		    makes radius to bail out..."
+		    
+		    hence the need for this code: */
+		
+		if (return_count) {
+			if (PQntuples(res) > 0 && PQnfields(res) > 0) 
+				*return_count = atoi(PQgetvalue(res, 0, 0));
+			else
+				*return_count = 0;
+		}
+		rc = 0;
+		break;
+
+	default:
+                grad_log(L_ERR,
+                         _("PQexec returned %s"),
+                         PQresStatus(stat));
                 if (stat == PGRES_FATAL_ERROR) 
 			rad_postgres_disconnect(conn, 0);
                 rc = -1;
@@ -146,9 +171,10 @@ rad_postgres_getpwd(struct sql_connection *conn, char *query)
         
         res = PQexec((PGconn*)conn->data, query);
         if (res == NULL) {
-                radlog(L_ERR,
-                       "PQexec: %s",
-                       PQerrorMessage((PGconn*)conn->data));
+                grad_log(L_ERR,
+                         "PQexec: %s",
+                         PQerrorMessage((PGconn*)conn->data));
+		rad_postgres_disconnect(conn, 0);
                 return NULL;
         }
         
@@ -161,18 +187,18 @@ rad_postgres_getpwd(struct sql_connection *conn, char *query)
         if (stat == PGRES_TUPLES_OK) {
                 int ntuples = PQntuples(res);
                 if (ntuples > 1 && PQnfields(res)) {
-                        radlog(L_NOTICE,
-			       ngettext("query returned %d tuple: %s",
-					"query returned %d tuples: %s",
-					ntuples),
-                               ntuples, query);
+                        grad_log(L_NOTICE,
+			         ngettext("query returned %d tuple: %s",
+				  	  "query returned %d tuples: %s",
+					  ntuples),
+                                 ntuples, query);
                 } else if (ntuples == 1) {
-                        return_passwd = estrdup(PQgetvalue(res, 0, 0));
+                        return_passwd = grad_estrdup(PQgetvalue(res, 0, 0));
                 }
         } else {
-                radlog(L_ERR,
-                       _("PQexec returned %s"),
-                       PQresStatus(stat));
+                grad_log(L_ERR,
+                         _("PQexec returned %s"),
+                         PQresStatus(stat));
                 if (stat == PGRES_FATAL_ERROR)
 			rad_postgres_disconnect(conn, 0);
         }
@@ -221,9 +247,10 @@ rad_postgres_exec(struct sql_connection *conn, char *query)
         
         res = PQexec((PGconn*)conn->data, query);
         if (res == NULL) {
-                radlog(L_ERR,
-                       "PQexec: %s",
-                       PQerrorMessage((PGconn*)conn->data));
+                grad_log(L_ERR,
+                         "PQexec: %s",
+                         PQerrorMessage((PGconn*)conn->data));
+		rad_postgres_disconnect(conn, 0);
                 return NULL;
         }
         
@@ -236,15 +263,15 @@ rad_postgres_exec(struct sql_connection *conn, char *query)
         if (stat != PGRES_TUPLES_OK) {
                 PQclear(res);
                 if (stat == PGRES_FATAL_ERROR) {
-			radlog(L_ERR,
-			       _("PQexec returned %s"),
-			       PQresStatus(stat));
+			grad_log(L_ERR,
+			         _("PQexec returned %s"),
+			         PQresStatus(stat));
 			rad_postgres_disconnect(conn, 0);
 		}
                 return NULL;
         }
 
-        data = emalloc(sizeof(*data));
+        data = grad_emalloc(sizeof(*data));
         data->res = res;
         data->ntuples = PQntuples(res);
         data->curtuple = -1;
@@ -288,10 +315,10 @@ rad_postgres_free(struct sql_connection *conn, void *data)
                 return;
         
         PQclear(edata->res);
-        efree(edata);
+        grad_free(edata);
 }
 
-SQL_DISPATCH_TAB postgres_dispatch_tab[] = {
+DECL_SQL_DISPATCH_TAB(postgres) = {
         "postgres",     
         5432,
         rad_postgres_reconnect,
@@ -306,5 +333,3 @@ SQL_DISPATCH_TAB postgres_dispatch_tab[] = {
 	rad_postgres_n_columns,
 };
 
-#endif
-
